'use strict';

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _yylibUi = require('yylib-ui');

var _ajax = require('yylib-utils/ajax');

var _ajax2 = _interopRequireDefault(_ajax);

var _jsCookie = require('js-cookie');

var _jsCookie2 = _interopRequireDefault(_jsCookie);

var _NccContext = require('../../utils/NccContext');

var _NccContext2 = _interopRequireDefault(_NccContext);

var _NccUrlUtils = require('../../utils/NccUrlUtils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * ncc联查凭证
 * @author mxx
 */
var field = _yylibUi.YYClass.field,
    PropTypes = _yylibUi.YYClass.PropTypes;


var uiMeta = {
    name: 'ncc-link-credentials',
    fields: [
    //field 参数列表：字段属性名|属性默认值|属性类型|是否必填|属性描述
    //------------联查按钮配置------------//
    field('type', 'error', PropTypes.oneOf(['primary', 'success', 'info', 'warning', 'error', '']), false, '设置按钮类型，可选值为 primary|success|info|warning|error 或者不设'), field('buttonText', '联查凭证', PropTypes.string, false, '设置按钮的文本'), field('icon', 'link', PropTypes.string, false, '设置按钮的图标类型'), field('visible', true, PropTypes.bool, false, '设置按钮是否显示'), field('disabled', false, PropTypes.bool, false, '设置按钮是否禁用'), field('ghost', true, PropTypes.bool, false, '设置按钮为不填充形态')
    //------------联查服务配置------------//
    , field('serverUrl', window.EnvConfig.PRINT_BASE_URL, PropTypes.string, true, 'ncc后端域名'), field('queryCredentials', '/nccloud/jzbase/jzweb/linkvouchar.do', PropTypes.string, true, '联查凭证地址'), field('appCode', '', PropTypes.string, true, 'ncc的应用编号'), field('pk_billtype', '', PropTypes.string, true, '单据类型'), field('relationID', '', PropTypes.string, false, '关联主键')]
};

var NccLinkCredentialsButton = _yylibUi.YYClass.create({
    uiMeta: uiMeta,
    displayName: 'NccLinkCredentialsButton',

    getInitialState: function getInitialState() {
        return {
            appCode: this.props.appCode,
            relationID: this.props.relationID,
            pk_billtype: this.props.pk_billtype,
            pk_group: _NccContext2.default.getGroupId()
            // pk_org: NccContext.getOrgId(),
        };
    },
    componentWillReceiveProps: function componentWillReceiveProps(nextProps) {
        //获取必要参数
        if ('appCode' in nextProps && nextProps.appCode !== this.props.appCode) {
            this.setState({ appCode: nextProps.appCode });
        }
        if ('relationID' in nextProps && nextProps.relationID !== this.props.relationID) {
            this.setState({ relationID: nextProps.relationID });
        }
        if ('pk_billtype' in nextProps && nextProps.pk_billtype !== this.props.pk_billtype) {
            this.setState({ pk_billtype: nextProps.pk_billtype });
        }
        if ('pk_group' in nextProps && nextProps.pk_group !== this.props.pk_group) {
            this.setState({ pk_group: nextProps.pk_group });
        }
    },
    onClick: function onClick() {
        var _props = this.props,
            serverUrl = _props.serverUrl,
            queryCredentials = _props.queryCredentials;
        var _state = this.state,
            appCode = _state.appCode,
            relationID = _state.relationID,
            pk_billtype = _state.pk_billtype,
            pk_group = _state.pk_group;

        if (!appCode || !relationID || !pk_billtype || !pk_group || !serverUrl || !queryCredentials) {
            _yylibUi.YYMessage.error("缺少必要参数");
            return;
        }
        //查询凭证
        var params = {
            busiParamJson: JSON.stringify([{
                pk_group: pk_group, //'0001A810000000000FK0',//
                // pk_org: pk_org,//'0001A810000000002QUK',//
                relationID: relationID, //'1001A810000000007LTC',//
                pk_billtype: pk_billtype //'H551-SK',//
            }]),
            sysParamJson: {
                appcode: appCode,
                pagecs: _jsCookie2.default.get("cookiets")
            }
        };
        _ajax2.default.postJSONWithCookies(serverUrl + queryCredentials, params, function (result) {
            if (result && result.success) {
                var _result$data = result.data,
                    appcode = _result$data.appcode,
                    pagecode = _result$data.pagecode,
                    url = _result$data.url,
                    pklist = _result$data.pklist;

                if (!pklist || !pklist.length) {
                    _yylibUi.YYMessage.error("未查询到凭证信息", 3);
                    return;
                }
                var _params = {
                    status: 'browse',
                    appcode: appcode,
                    pagecode: pagecode,
                    id: pklist[0],
                    n: '联查凭证',
                    backflag: 'noback',
                    serverUrl: serverUrl
                };
                (0, _NccUrlUtils.openTo)(url, _params);
            } else {
                _yylibUi.YYMessage.error("联查凭证失败");
            }
        }, function () {
            _yylibUi.YYMessage.error("联查凭证失败");
        });
    },
    render: function render() {
        var _props2 = this.props,
            visible = _props2.visible,
            type = _props2.type,
            icon = _props2.icon,
            buttonText = _props2.buttonText,
            disabled = _props2.disabled,
            ghost = _props2.ghost;

        return _react2.default.createElement(
            'div',
            { className: this.className(), style: { display: 'inline-block' } },
            _react2.default.createElement(
                _yylibUi.YYButton,
                { icon: icon, type: type, ghost: ghost, disabled: disabled, visible: visible,
                    onClick: this.onClick },
                buttonText
            )
        );
    }
});
module.exports = NccLinkCredentialsButton;