'use strict';

var _ = require('lodash');
var superagent = require('superagent');
var jsonp = require('superagent-jsonp');
var Cookies = require('js-cookie');
var BaseHost = window.EnvConfig.ADDR;
var getWorkbenchCookie = '/icop-workbench/usercontext/get';
var AuthToken = {
	CONTEXT_KEY: 'icop_context' //cookie键
	, SERVER_URL: window.EnvConfig.AUTHTOKE_HOST_URL //获取上下文的URL
	, _cacheContext: null
	/**
  * 初始化上下文信息
  * @param options 参数配置{
     *  url:获取上下文的URL，string|array
     *  success:初始化上下文成功时的回调
     *  error:初始化上下文失败时的回调
     * }
  */
	, init: function init(options) {
		var _url = options && options.url ? options.url : this.SERVER_URL; //无URL参数则使用默认获取上下文的URL
		var win = window.top || window;
		var _success = options && options.success ? options.success : null;
		var _error = options && options.error ? options.error : null;
		var _clear = options && options.clear !== undefined ? options.clear : true; //是否清除历史上下文
		if (_clear) {
			delete this._cacheContext; //清除内存级存储
		}
		var _this = this;
		var firstResp = false;
		if (window.parent["icop_usercontext"] && window.parent["icop_usercontext"]["u_usercode"]) {
			this._cacheContext = encodeURI(JSON.stringify(window.parent["icop_usercontext"]));
			this.UserContextStore(this._cacheContext, "add");
			_success();
		} else {
			superagent.get(_url).use(jsonp).end(function (err, res) {
				if (res != null && res.body != null && res.body != undefined) {
					_this._cacheContext = encodeURI(JSON.stringify(res.body));
					_this.UserContextStore(_this._cacheContext, "add");

					if (typeof _success == 'function') {
						_success();
					}
				} else {
					if (_clear) {
						delete _this._cacheContext; //清除内存级存储
						_this.UserContextStore(null, "delete"); //清除存储
					}
					if (firstResp == false) {
						firstResp = true; //解决错误回调被执行了两次的问题
					} else {
						if (typeof _error == 'function') {
							_error(err);
						}
					}
				}
			});
		}
	}
	/**
  * 获取上下文信息
  * @returns {String}
  */
	, getContext: function getContext() {
		if (!this._cacheContext) {
			//内存级存储
			var context = this.UserContextStore(null, "get"); //cookies级存储
			if (isJsonStr(context)) {
				//确保为JSON格式的字符串
				var jsTxt = '(' + decodeURIComponent(context.replace(/'/, "\'")) + ')';
				var jsonObj = eval(jsTxt); //转换为JSON对象
				this.newCacheContext = jsonObj;
			}
		} else {
			this.newCacheContext = JSON.parse(decodeURIComponent(this._cacheContext));
		}
		return this.newCacheContext;
	}, getToken: function getToken() {
		// 获取Token
		var context = this.getContext();
		if (context == null || context == undefined) {
			return null;
		}
		return context["token"];
	}, getStaff: function getStaff() {
		var context = this.getContext();
		if (context == null || context == undefined) {
			return null;
		}
		return {
			id: context["staffId"]
		};
	}, getStaffId: function getStaffId() {
		var context = this.getContext();
		if (context == null || context == undefined) {
			return null;
		}
		return context["staffId"];
	}, getDept: function getDept() {
		var context = this.getContext();
		if (context == null || context == undefined) {
			return null;
		}
		return {
			id: context["deptId"],
			code: context["deptCode"],
			name: context["deptName"]
		};
	}, getDeptId: function getDeptId() {
		var context = this.getContext();
		if (context == null || context == undefined) {
			return null;
		}
		return context["deptId"];
	}, getDeptCode: function getDeptCode() {
		var context = this.getContext();
		if (context == null || context == undefined) {
			return null;
		}
		return context["deptCode"];
	}, getDeptName: function getDeptName() {
		var context = this.getContext();
		if (context == null || context == undefined) {
			return null;
		}
		return context["deptName"];
	}, getUser: function getUser() {
		var context = this.getContext();
		if (context == null || context == undefined) {
			return null;
		}
		return {
			id: context["userId"],
			code: context["userCode"],
			name: context["userName"]
		};
	}, getUserId: function getUserId() {
		// 获取用户ID
		var context = this.getContext();
		if (context == null || context == undefined) {
			return null;
		}
		return context["userId"];
	}, getUserCode: function getUserCode() {
		// 获取用户编码
		var context = this.getContext();
		if (context == null || context == undefined) {
			return null;
		}
		return context["userCode"];
	}, getUserName: function getUserName() {
		// 获取用户名称
		var context = this.getContext();
		if (context == null || context == undefined) {
			return null;
		}
		return context["userName"];
	}, getOrga: function getOrga() {
		// 获取组织ID,CODE,NAME信息
		var context = this.getContext();
		if (context == null || context == undefined) {
			return null;
		}
		return {
			id: context["companyId"],
			code: context["companyCode"],
			name: context["companyName"]
		};
	},
	getOrgaId: function getOrgaId() {
		// 获取组织ID
		var context = this.getContext();
		if (context == null || context == undefined) {
			return null;
		}
		return context["companyId"];
	}, getOrgaCode: function getOrgaCode() {
		// 获取组织编码
		var context = this.getContext();
		if (context == null || context == undefined) {
			return null;
		}
		return context["companyCode"];
	}, getOrgaName: function getOrgaName() {
		// 获取组织名称
		var context = this.getContext();
		if (context == null || context == undefined) {
			return null;
		}
		return context["companyName"];
	}, getLogints: function getLogints() {
		var context = this.getContext();
		if (context == null || context == undefined) {
			return null;
		}
		return context["u_logints"];
	}, getTenantId: function getTenantId() {
		var context = this.getContext();
		if (context == null || context == undefined) {
			return null;
		}

		return context["tenantid"];
	}, getAuthenticationStr: function getAuthenticationStr() {
		//获取认证需要的字段信息
		var includeFields = ["u_logints", "u_usercode", "token", "tenantid", "userId", "userType", //认证需要的字段，下面为扩展字段
		"userName", "userCode", "userType", "companyId", "companyName", "companyCode"];
		var authentication = '';
		//从本域名下的cookies获取相关认证字段信息
		// includeFields.forEach(field=>{
		//     let fieldVal = Cookies.get(field);
		//     if(fieldVal){
		//         authentication += field + '=' + encodeURI(fieldVal) + ';';
		//     }
		// });
		// if(!authentication){
		//从cookies上下文icon_context中获取相关认证字段信息
		var set = new Set();
		_.forEach(includeFields, function (value) {
			set.add(value);
		});
		var context = this.getContext();
		if (context) {
			for (var prop in context) {
				if (set.has(prop) && context[prop]) {
					authentication += prop + '=' + encodeURI(context[prop]) + ';';
				}
			}
		}
		// }
		if (authentication.length > 0 && authentication[authentication.length - 1] == ';') authentication = authentication.substring(0, authentication.length - 1);
		return authentication;
	},
	getValidation: function getValidation() {
		var context = this.getContext();
		if (context == null || context == undefined) {
			return null;
		}
		return context["validation"];
	},
	UserContextStore: function UserContextStore(cacheContext, type) {
		if ("add" === type) {
			if (window.sessionStorage) {
				sessionStorage.setItem(this.CONTEXT_KEY, cacheContext);
			} else {
				Cookies.set(this.CONTEXT_KEY, cacheContext);
			}
		} else if ("delete" === type) {
			if (window.sessionStorage) {
				sessionStorage.removeItem(this.CONTEXT_KEY);
			} else {
				Cookies.remove(this.CONTEXT_KEY);
			}
		} else if ("get" === type) {
			if (window.sessionStorage) {
				return sessionStorage.getItem(this.CONTEXT_KEY);
			} else {
				return Cookies.get(this.CONTEXT_KEY);
			}
		}
	}
};

/**
 * @desc 是否为JSON对象格式的字符串形态。匹配格式:"{...}"
 */
function isJsonStr(val) {
	return typeof val == "string" && /^\{.*\}$/.test(val);
}
var UrlUtils = {
	/**
  *填充url参数，通过参数key值替换 "{}" 里面的内容
  *@param url "http://xxxx/crm/{userId}" 或者 "http://xxxx/crm/list?userId={userId}"
  *@param params {userId: "001"}
  *@return "http://xxxx/crm/001" 或者 "http://xxxx/crm/list?userId=001"
  */
	fillUrlParams: function fillUrlParams(url, params) {
		if (_.isString(url) && _.isObject(params)) {
			var retUrl = url;
			_.forIn(params, function (value, key) {
				retUrl = retUrl.replace(new RegExp('{' + encodeURI(key) + '}', 'gm'), encodeURI(value));
			});
			return retUrl;
		}
		return url;
	}
	/**
  * @desc 将参数追加到URL后面,支持中文参数
  * @param params{object}需要追加的参数
  * @return 返回追加参数后的URL
  */
	, appendToUrl: function appendToUrl(url, params) {
		var _url = url;
		var isFirstParam = true; //是否为URL的第一个参数
		if (_url.indexOf("?") == -1) {
			//URL是否存在"?"
			_url += "?";
		} else {
			isFirstParam = false;
		}
		var _params = this.getParams(params);
		if (_.isPlainObject(_params) && !_.isEmpty(_params)) {
			_.forIn(_params, function (val, key) {
				var _val = val == undefined ? '' : val;
				if (isFirstParam) {
					isFirstParam = false;
					_url += key + "=" + _val;
				} else {
					var _matchReg = new RegExp("[\?|&]" + key + "=[^&]*");
					var _replaceReg = new RegExp(key + "=[^&]*");
					if (_url.match(_matchReg)) {
						//存在相同Key值则进行替换
						_url = _url.replace(_replaceReg, key + "=" + _val);
					} else {
						_url += "&" + key + "=" + _val;
					}
				}
			});
		}
		return _url;
	}
	/**
  * @desc 解析动态参数获取最终参数值
  * @param 支持以下两类格式：
  * --------Function---------
  * function(){
     *   return {myKey1:'myVal1',myKey2:'myVal2'}
     * }
  * --------Obejct内嵌Function（一级）---------
  * {
     *    key0:"value0"
     *    ,key1:function(){
     *      return {key11:"value11"}
     *    }
     * }
  * 当参数中存在function，会执行并获取其值。
  * @return 最终返回简单格式的JSON对象
  */
	, getParams: function getParams(param) {
		var _otherParam = {};
		if (_.isFunction(param)) {
			return param.call();
		} else if (_.isPlainObject(param)) {
			_.forIn(param, function (propValue, propName) {
				if (_.isFunction(propValue)) {
					_otherParam[propName] = propValue.call();
				} else {
					_otherParam[propName] = propValue;
				}
			});
			return _otherParam;
		}
		return {};
	}
	/**
  * 解析标准的URL
  *  var myURL = parseURL('http://abc.com:8080/dir/index.html?id=255&m=hello#top');
  *  myURL.file = 'index.html'
  *  myURL.hash = 'top'
  *  myURL.host = 'abc.com'
  *  myURL.query = '?id=255&m=hello'
  *  myURL.params = Object = { id: 255, m: hello }
  *  myURL.path = '/dir/index.html'
  *  myURL.segments = Array = ['dir', 'index.html']
  *  myURL.port = '8080'
  *  myURL.protocol = 'http'
  *  myURL.relative = '/dir/index.html?id=255&m=hello#top'
  *  myURL.source = 'http://abc.com:8080/dir/index.html?id=255&m=hello#top'
  * @param url
  * @returns {{source: *, protocol, host, port, query, params, file: string, hash, path, relative: string, segments}}
  */
	, parseURL: function parseURL(url) {
		var a = document.createElement('a');
		a.href = url;
		return {
			source: url,
			protocol: a.protocol.replace(':', ''),
			host: a.hostname,
			port: a.port,
			query: a.search,
			params: function () {
				var ret = {},
				    seg = a.search.replace(/^\?/, '').split('&'),
				    len = seg.length,
				    i = 0,
				    s;
				for (; i < len; i++) {
					if (!seg[i]) {
						continue;
					}
					s = seg[i].split('=');
					ret[s[0]] = s[1];
				}
				return ret;
			}(),
			file: (a.pathname.match(/\/([^\/?#]+)$/i) || [, ''])[1],
			hash: a.hash.replace('#', ''),
			path: a.pathname.replace(/^([^\/])/, '/$1'),
			relative: (a.href.match(/tps?:\/\/[^\/]+(.+)/) || [, ''])[1],
			segments: a.pathname.replace(/^\//, '').split('/')
		};
	},
	getBaseUrl: function getBaseUrl() {
		var urlObj = this.parseURL(document.URL);
		return urlObj.protocol + '://' + urlObj.host + ':' + urlObj.port;
	}

};

module.exports = AuthToken;