'use strict';

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _yylibUi = require('yylib-ui');

var _antd = require('antd');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * 订阅
 * @author maxx4
 */
var field = _yylibUi.YYClass.field,
    PropTypes = _yylibUi.YYClass.PropTypes;

var ajax = require('yylib-utils/ajax');
var AuthToken = require("../../utils/AuthToken");


var Option = _antd.Select.Option;
var YYConfirm = _yylibUi.YYModal.confirm;

var queryTaskUrl = window.EnvConfig.ADDR + "/icop-earlywarn-web/subsTask/query";
var saveTaskUrl = window.EnvConfig.ADDR + "/icop-earlywarn-web/subsTask/save";
var removeTaskUrl = window.EnvConfig.ADDR + "/icop-earlywarn-web/subsTask/remove";

var uiMeta = {
    name: 'subsbutton',
    fields: [
    //field 参数列表：字段属性名|属性默认值|属性类型|是否必填|属性描述
    //------------订阅按钮配置------------//
    field('type', 'error', PropTypes.oneOf(['primary', 'success', 'info', 'warning', 'error', '']), false, '设置按钮类型，可选值为 primary|success|info|warning|error 或者不设'), field('buttonText', '订阅', PropTypes.string, false, '设置按钮的文本'), field('icon', 'warning', PropTypes.string, false, '设置按钮的图标类型'), field('visible', true, PropTypes.bool, false, '设置按钮是否显示'), field('disabled', false, PropTypes.bool, false, '设置按钮是否禁用'), field('ghost', true, PropTypes.bool, false, '设置按钮为不填充形态')]
};

var SubsButton = _yylibUi.YYClass.create({
    uiMeta: uiMeta,
    displayName: 'SubsButton',

    getInitialState: function getInitialState() {
        return {
            showModal: false,
            subsType: "week", //默认周订阅
            subsColumn: [], //订阅字段
            subsInfo: null, //订阅信息
            title: this.props.title, //订阅标题
            remark: "", //订阅备注
            subsDate: "lw" //默认上周
        };
    },
    componentWillMount: function componentWillMount() {
        if (this.props.setColumns) {
            //设置订阅字段
            this.props.setColumns();
        }
    },
    onClick: function onClick() {
        //this.setState({showModal: true});
        var _props = this.props,
            onlyKey = _props.onlyKey,
            serverUrl = _props.serverUrl,
            columns = _props.columns,
            subsSubType = _props.subsSubType;

        if (!onlyKey) {
            _yylibUi.YYMessage.error("唯一标识为空！");
            return;
        }

        if (!serverUrl) {
            _yylibUi.YYMessage.error("服务地址为空！");
            return;
        }

        if (!columns || columns.length === 0) {
            _yylibUi.YYMessage.error("订阅列为空！");
            return;
        }

        if (!subsSubType) {
            _yylibUi.YYMessage.error("订阅类型为空！");
            return;
        }

        //预留校验接口，如果不满足校验，不弹出订阅接口
        if (this.props.verify) {
            var isVerify = this.props.verify();
            if (isVerify && isVerify === false) {
                return;
            }
        }

        var that = this;
        //获取已经订阅信息
        ajax.getJSON(queryTaskUrl + ('?subsKey=' + onlyKey), null, function (data) {
            if (data && data.success) {
                //已经订阅过了
                var _data = {};
                if (data.backData) {
                    _data.subsType = data.backData.period;
                    var subTemplate = JSON.parse(data.backData.subTemplate);
                    _data.subsInfo = data.backData;
                    _data.title = data.backData.subTitle;
                    _data.remark = data.backData.remark;
                    _data.subsColumn = subTemplate.map(function (item) {
                        return item.value;
                    });
                } else {
                    _data.remark = "";
                    _data.subsColumn = [];
                    _data.subsInfo = null;
                }
                _data.showModal = true;
                that.setState(_data);
            } else {
                _yylibUi.YYMessage.error('\u67E5\u8BE2\u8BA2\u9605\u4FE1\u606F\u5931\u8D25\uFF01' + backMsg);
            }
        });
    },
    onCancel: function onCancel() {
        this.setState({ showModal: false });
    },
    onCancelSubs: function onCancelSubs() {
        var subsInfo = this.state.subsInfo;

        if (subsInfo !== null) {
            //已经订阅过相应的任务
            var taskId = subsInfo.id; //任务主键
            ajax.postJSON(removeTaskUrl, { id: taskId }, function (data) {
                if (data && data.success) {
                    _yylibUi.YYMessage.success('\u53D6\u6D88\u8BA2\u9605\u6210\u529F\uFF01');
                } else {
                    _yylibUi.YYMessage.error('\u53D6\u6D88\u8BA2\u9605\u5931\u8D25\uFF01' + data.backMsg);
                }
            });
        }
        this.setState({ showModal: false });
    },


    //确认订阅
    onOkSubs: function onOkSubs() {
        //订阅周期，订阅字段，订阅条件
        var _props2 = this.props,
            onlyKey = _props2.onlyKey,
            serverUrl = _props2.serverUrl,
            columns = _props2.columns,
            subsSubType = _props2.subsSubType,
            subsDate = _props2.subsDate;
        var _state = this.state,
            subsType = _state.subsType,
            subsInfo = _state.subsInfo,
            title = _state.title,
            subsColumn = _state.subsColumn;


        if (!title) {
            _yylibUi.YYMessage.error("订阅标题为空！");
            return;
        }

        //构造查询条件
        //获取查询条件
        var conditions = null;
        if (this.props.getConditons) {
            conditions = this.props.getConditons(subsDate);
        }

        var that = this;
        if (!conditions) {
            //是否继续
            YYConfirm({
                title: '查询条件!',
                content: '请注意没有预制订阅条件，如果确认继续订阅，如取消请联系研发处理！',
                onOk: function onOk() {
                    that.saveTask(conditions, subsType, serverUrl, onlyKey, title, subsSubType, subsColumn, columns, subsInfo);
                }
            });
        } else {
            this.saveTask(conditions, subsType, serverUrl, onlyKey, title, subsSubType, subsColumn, columns, subsInfo);
        }
    },
    saveTask: function saveTask(conditions, subsType, serverUrl, onlyKey, title, subsSubType, subsColumn, columns, subsInfo) {
        var params = {};
        params.conditions = conditions;
        params.period = subsType;
        params.subAddr = serverUrl;
        params.subsKey = onlyKey;
        params.subTitle = title;
        params.subType = subsSubType;
        params.companyId = AuthToken.getOrgaId(); //组织
        params.subscriber = AuthToken.getUserId(); //用户
        params.subscriberCode = AuthToken.getUserCode(); //用户编码

        var _subTemplate = columns.filter(function (item) {
            return subsColumn.indexOf(item.value) > -1;
        });
        params.subTemplate = JSON.stringify(_subTemplate);

        if (subsInfo !== null) {
            //修改
            params.id = subsInfo.id;
        }

        ajax.postJSON(saveTaskUrl, params, function (data) {
            if (data && data.success) {
                _yylibUi.YYMessage.success('\u8BA2\u9605\u6210\u529F\uFF01');
            } else {
                _yylibUi.YYMessage.error('\u8BA2\u9605\u5931\u8D25\uFF01' + data.backMsg);
            }
        });
        this.setState({ showModal: false });
    },
    onRadioChange: function onRadioChange(e) {
        var subsType = e.target.value;
        this.setState({
            subsType: subsType
        });
    },
    onTimeRadioChange: function onTimeRadioChange(e) {
        var subsDate = e.target.subsDate;
        this.setState({
            subsDate: subsDate
        });
    },
    changeTitle: function changeTitle(e, m) {
        this.setState({
            title: e.target.value
        });
    },
    handleChange: function handleChange(e) {
        if (!e || e.length === 0) {
            this.setState({ subsColumn: [] });
        }
        this.setState({ subsColumn: e });
    },
    render: function render() {
        var _props3 = this.props,
            visible = _props3.visible,
            type = _props3.type,
            icon = _props3.icon,
            buttonText = _props3.buttonText,
            disabled = _props3.disabled,
            ghost = _props3.ghost,
            columns = _props3.columns;
        var _state2 = this.state,
            showModal = _state2.showModal,
            subsType = _state2.subsType,
            subsColumn = _state2.subsColumn,
            title = _state2.title,
            subsDate = _state2.subsDate;

        var width = document.documentElement.clientWidth * 0.8;
        var height = document.documentElement.clientHeight * 0.6;

        var children = [];
        if (columns && columns.length > 0) {
            columns.map(function (item) {
                children.push(_react2.default.createElement(
                    Option,
                    { key: item.value },
                    item.name
                ));
            });
        }
        return _react2.default.createElement(
            'div',
            { className: this.className(), style: { display: 'inline-block' } },
            _react2.default.createElement(
                _yylibUi.YYButton,
                { icon: icon, type: type, ghost: ghost, disabled: disabled, visible: visible, style: { marginTop: "6px" },
                    onClick: this.onClick },
                buttonText
            ),
            showModal === true ? _react2.default.createElement(
                _yylibUi.YYModal,
                { title: '订阅设置', visible: showModal, width: width,
                    maskClosable: false, bodyStyle: { padding: 0 },
                    onCancel: this.onCancel, style: { top: height * 0.1 },
                    footer: _react2.default.createElement(
                        'div',
                        { style: { textAlign: "center" } },
                        _react2.default.createElement(
                            _yylibUi.YYButton,
                            { type: 'primary', onClick: this.onOkSubs },
                            '\u786E\u8BA4\u8BA2\u9605'
                        ),
                        _react2.default.createElement(
                            _yylibUi.YYButton,
                            { type: 'primary', onClick: this.onCancelSubs },
                            '\u53D6\u6D88\u8BA2\u9605'
                        )
                    )
                },
                _react2.default.createElement(
                    'div',
                    { style: { margin: "10px 20px" } },
                    _react2.default.createElement(
                        'div',
                        { style: {
                                marginBottom: "10px",
                                fontWeight: "700"
                            } },
                        '\u8BA2\u9605\u6807\u9898\uFF1A'
                    ),
                    _react2.default.createElement(_yylibUi.YYInput, { value: title, onChange: this.changeTitle })
                ),
                _react2.default.createElement(
                    'div',
                    { style: { margin: "0 20px 10px 20px" } },
                    _react2.default.createElement(
                        'div',
                        { style: {
                                marginBottom: "10px",
                                fontWeight: "700"
                            } },
                        '\u8BA2\u9605\u5468\u671F\uFF1A'
                    ),
                    _react2.default.createElement(
                        _yylibUi.YYRadioGroup,
                        { onChange: this.onRadioChange, defaultValue: subsType, value: subsType },
                        _react2.default.createElement(
                            _yylibUi.YYRadio,
                            { value: "week", uititle: "周订阅", uikey: "week" },
                            '\u5468\u8BA2\u9605 ',
                            _react2.default.createElement(
                                _antd.Tooltip,
                                {
                                    placement: 'bottom', title: "每周一推送订阅内容" },
                                _react2.default.createElement(_antd.Icon, { type: 'question-circle-o' })
                            )
                        ),
                        _react2.default.createElement(
                            _yylibUi.YYRadio,
                            { value: "month", uititle: "月订阅", month: "month" },
                            '\u6708\u8BA2\u9605 ',
                            _react2.default.createElement(
                                _antd.Tooltip,
                                {
                                    placement: 'bottom', title: "每月1号推送订阅内容" },
                                _react2.default.createElement(_antd.Icon, { type: 'question-circle-o' })
                            )
                        )
                    )
                ),
                _react2.default.createElement(
                    'div',
                    { style: { margin: "0 20px 10px 20px" } },
                    _react2.default.createElement(
                        'div',
                        { style: {
                                marginBottom: "10px",
                                fontWeight: "700"
                            } },
                        '\u65F6\u95F4\u8303\u56F4\uFF1A'
                    ),
                    _react2.default.createElement(
                        _yylibUi.YYRadioGroup,
                        { onChange: this.onTimeRadioChange, defaultValue: subsDate,
                            value: subsDate },
                        _react2.default.createElement(
                            _yylibUi.YYRadio,
                            { value: "lw", uititle: "上周", uikey: "lw" },
                            '\u4E0A\u5468'
                        ),
                        _react2.default.createElement(
                            _yylibUi.YYRadio,
                            { value: "tw", uititle: "本周", month: "tw" },
                            '\u672C\u5468'
                        ),
                        _react2.default.createElement(
                            _yylibUi.YYRadio,
                            { value: "lm", uititle: "上月", month: "tw" },
                            '\u4E0A\u6708'
                        ),
                        _react2.default.createElement(
                            _yylibUi.YYRadio,
                            { value: "tm", uititle: "本月", month: "tw" },
                            '\u672C\u6708'
                        ),
                        _react2.default.createElement(
                            _yylibUi.YYRadio,
                            { value: "ty", uititle: "本年", month: "tw" },
                            '\u672C\u5E74'
                        )
                    )
                ),
                _react2.default.createElement(
                    'div',
                    { style: { margin: "0 20px 10px 20px" } },
                    _react2.default.createElement(
                        'div',
                        { style: {
                                marginBottom: "10px",
                                fontWeight: "700"
                            } },
                        '\u8BA2\u9605\u5B57\u6BB5\uFF1A'
                    ),
                    children && children.length > 0 && _react2.default.createElement(
                        _antd.Select,
                        {
                            style: { width: '100%' },
                            multiple: true,
                            defaultValue: subsColumn,
                            placeholder: '\u8BF7\u9009\u62E9\u8BA2\u9605\u5217',
                            onChange: this.handleChange
                        },
                        children
                    )
                )
            ) : null
        );
    }
});
module.exports = SubsButton;