'use strict';

/**
 * NCC表体 互算功能
 * @type {{}}
 */
var Cookies = require('js-cookie');
var ajax = require('yylib-utils/ajax');
var _ui = require('yylib-ui/');
var YYMessage = _ui.YYMessage;
var ADDR = window.EnvConfig.ADDR;
var DataUtil = require('yylib-utils//DataUtil');
var NccContext = require('yylib-utils/NccContext');
var REQUESTURL = "/nccloud/jzbase/jzweb/BizParam.do";
var CACHE_KEY = 'BUSINESS_PARAM_';

var fractionDigits = 8; // 小数精度
var _taxFirst = true; // 含税优先
var _priceFirst = true; // 单价优先

var money = NccContext.getScales().money;
var num = NccContext.getScales().num;
var price = NccContext.getScales().price;
var rate = NccContext.getScales().rate;

var NccOtherCalculate = {
    //获取含税优先，还是无税优先
    getTaxFirst: function getTaxFirst(params, callback) {

        //是否在缓存中

        var url = ADDR + REQUESTURL;
        ajax.postJSONWithCookies(url, params, function (result) {
            if (result && result.success) {
                callback(result.data);
                //缓存
            } else {
                YYMessage.error(result.error && result.error.message ? result.error.message : '查询失败');
            }
        });
    },

    /**
     *
     * @param page
     * @param editApi
     * @param pk_org //组织 ，获取含税优先还是无税优先
     * @param key //当前变化的key
     * @param rowKey //行键值
     * @param newValue //新值
     * @param priceFirst //是否是单价优先
     * @param fileds  //需要处理的字段
     * @param headOrBody  //需要处理的字段
     * @param callback
     */
    setValueByEditField: function setValueByEditField(_ref) {
        var _this = this;

        var page = _ref.page,
            code = _ref.code,
            pk_org = _ref.pk_org,
            key = _ref.key,
            rowKey = _ref.rowKey,
            newValue = _ref.newValue,
            priceFirst = _ref.priceFirst,
            fileds = _ref.fileds,
            headOrBody = _ref.headOrBody,
            callback = _ref.callback;

        var params = this.getParam(pk_org);
        //是否单价优先
        if (!priceFirst) {
            _priceFirst = priceFirst;
        }
        //是否无税优先
        var _bussinessParam = DataUtil.getCache(CACHE_KEY + pk_org);

        //获取精度
        money = NccContext.getScales().money;
        num = NccContext.getScales().num;
        price = NccContext.getScales().price;
        rate = NccContext.getScales().rate;

        //存在缓存
        if (_bussinessParam) {
            _taxFirst = _bussinessParam;
            this.calculate(page, priceFirst, _taxFirst, headOrBody, code, key, rowKey, newValue, fileds);
        } else {
            this.getTaxFirst(params, function (result) {
                //是否含税优先
                _taxFirst = !(result && result['JZ003'] && result['JZ003'] === '无税优先');
                DataUtil.setCache(CACHE_KEY + pk_org, _taxFirst);
                _this.calculate(page, priceFirst, _taxFirst, headOrBody, code, key, rowKey, newValue, fileds);
            });
        }

        //互算完成之后的操作，类似于汇总表头等...
        callback && callback();
    },

    calculate: function calculate(page, priceFirst, _taxFirst, headOrBody, code, key, rowKey, newValue, fileds) {
        fileds = this.dealRelationFiled(fileds);
        if (headOrBody === 'body') {
            var editTableApi = page.findUI(code).api;
            this.setValueByEditBodyField(editTableApi, key, rowKey, newValue, fileds);
        }
        if (headOrBody === 'head') {
            var formApi = page.findUI(code).api;
            this.setValueByEditHeadField(formApi, key, newValue, fileds);
        }
    },

    getParam: function getParam(pk_org) {
        var params = {
            busiParamJson: "{\"pk_org\":\"" + pk_org + "\",\"code\":\"JZ003\"}",
            sysParamJson: {
                pagecs: Cookies.get("cookiets"),
                appcode: '10228888',
                pk_org: pk_org,
                code: 'JZ003'
            }
        };
        return params;
    },

    /**
     * @param editTableApi
     * @param key
     * @param rowKey
     * @param newValue
     * @param fileds
     */
    setValueByEditBodyField: function setValueByEditBodyField(editTableApi, key, rowKey, newValue, fileds) {
        var pk_taxrate = fileds.pk_taxrate,
            ntaxrate = fileds.ntaxrate,
            nnum = fileds.nnum,
            nprice = fileds.nprice,
            ntaxprice = fileds.ntaxprice,
            norigmny = fileds.norigmny,
            norigbilltaxmny = fileds.norigbilltaxmny,
            norigtaxmny = fileds.norigtaxmny,
            nmny = fileds.nmny,
            nbilltaxmny = fileds.nbilltaxmny,
            ntaxmny = fileds.ntaxmny;
        // 清空按0处理?

        if (!newValue && key !== pk_taxrate) {
            editTableApi.saveCellData(rowKey, key, 0);
        }
        switch (key) {
            case pk_taxrate: // 税率变更
            case ntaxrate:
                // 税率变更
                this.setAllMnyByTaxRate(editTableApi, fileds, rowKey);
                break;
            case nnum:
                // 数量变更
                this.setAllMnyByNum(editTableApi, fileds, rowKey);
                break;
            case nprice:
                // 单价无税变更
                this.setAllMnyByPrice(editTableApi, fileds, rowKey);
                break;
            case ntaxprice:
                // 单价含税变更
                this.setAllMnyByPriceTax(editTableApi, fileds, rowKey);
                break;
            case norigmny:
                // 金额无税变更
                this.setAllMnyByAmountMny(editTableApi, fileds, rowKey);
                break;
            case norigbilltaxmny:
                // 金额含税变更
                this.setAllMnyByAmountTaxMny(editTableApi, fileds, rowKey);
                break;
            case norigtaxmny:
                // 税额变更
                this.setAllMnyByTaxMny(editTableApi, fileds, rowKey);
                break;
            default:
                return;
        }
    },

    /**
     *
     * @param editFormApi
     * @param key
     * @param newValue
     * @param fileds
     */
    setValueByEditHeadField: function setValueByEditHeadField(editFormApi, key, newValue, fileds) {
        var pk_taxrate = fileds.pk_taxrate,
            ntaxrate = fileds.ntaxrate,
            nnum = fileds.nnum,
            nprice = fileds.nprice,
            ntaxprice = fileds.ntaxprice,
            norigmny = fileds.norigmny,
            norigbilltaxmny = fileds.norigbilltaxmny,
            norigtaxmny = fileds.norigtaxmny,
            nmny = fileds.nmny,
            nbilltaxmny = fileds.nbilltaxmny,
            ntaxmny = fileds.ntaxmny;
        // 清空按0处理?

        if (!newValue && key !== pk_taxrate) {
            var _data = {};
            _data[key] = newValue;
            editFormApi.setFieldsValue(_data);
        }
        switch (key) {
            case pk_taxrate: // 税率变更
            case ntaxrate:
                // 税率变更
                this.setHeadAllMnyByTaxRate(editFormApi, fileds);
                break;
            case nnum:
                // 数量变更
                this.setHeadAllMnyByNum(editFormApi, fileds);
                break;
            case nprice:
                // 单价无税变更
                this.setHeadAllMnyByPrice(editFormApi, fileds);
                break;
            case ntaxprice:
                // 单价含税变更
                this.setHeadAllMnyByPriceTax(editFormApi, fileds);
                break;
            case norigmny:
                // 金额无税变更
                this.setHeadAllMnyByAmountMny(editFormApi, fileds);
                break;
            case norigbilltaxmny:
                // 金额含税变更
                this.setHeadAllMnyByAmountTaxMny(editFormApi, fileds);
                break;
            case norigtaxmny:
                // 税额变更
                this.setHeadAllMnyByTaxMny(editFormApi, fileds);
                break;
            default:
                return;
        }
    },

    /**
     * "税率"变更
     * 编辑表头"税率"字段，根据参照“含税优先/无税优先”，设置单价和金额字段；
     * 如果“含税优先”，设置“单价(无税)”和“税额”、“税额(本币)”、“金额(无税)、“金额(无税)(本币)”字段值；
     * 如果“无税优先”，设置“单价”和“税额”、“税额(本币)”、“金额”、“金额(本币)”字段值
     * @param editTableApi
     * @param fileds
     * @param rowKey
     */
    setAllMnyByTaxRate: function setAllMnyByTaxRate(editTableApi, fileds, rowKey) {
        if (_taxFirst) {
            // 含税优先
            this.setNotTaxPriceAndAllMnyByTaxRate(editTableApi, fileds, rowKey);
        } else {
            // 无税优先
            this.setHasTaxPriceAndAllMnyByTaxRate(editTableApi, fileds, rowKey);
        }
    },

    /**
     * "税率"变更-无税优先
     *
     * 编辑表头"税率"字段，设置“单价”和“税额”、“税额(本币)”、“金额”、“金额(本币)”字段值
     * 1)    如果有“单价(无税)”和“单价”字段，则设置
     单价=单价(无税)*(1+税率(%)/100)
     2)    如果有“金额(无税)”和“金额”，则设置
     金额= 单价*数量
     3)    如果有“税额”字段，则设置：
     税额= 金额(无税)* 税率(%)/100
     ii.    如果无“单价(无税)”和“单价”字段，但有“金额(无税)”和“金额”字段，则设置
     1)    金额=金额(无税)*(1+税率(%)/100)
     2)    如果有“税额”字段，则设置：税额= 金额(无税)* 税率(%)/100
     * @param editTableApi
     * @param fileds
     * @param rowKey
     */
    setHasTaxPriceAndAllMnyByTaxRate: function setHasTaxPriceAndAllMnyByTaxRate(editTableApi, fileds, rowKey) {
        this.calcTaxPrice(editTableApi, fileds, rowKey);
        this.calcTaxMny(editTableApi, fileds, rowKey);
        this.calcTaxMny2(editTableApi, fileds, rowKey);
        this.calcMnyTax(editTableApi, fileds, rowKey);
    },

    /**
     * "税率"变更-含税优先
     *
     * 编辑表头"税率"字段，设置“单价(无税)”和“税额”、“税额(本币)”、“金额(无税)、“金额(无税)(本币)”字段值
     * 1)    如果有“单价(无税)”和“单价”字段，则设置
     单价(无税)=单价/(1+税率(%)/100)
     2)    如果有“金额(无税)”和“金额”，则设置
     金额(无税)= 单价(无税)*数量
     3)    如果有“税额”字段，则设置：
     税额= 金额(无税)* 税率(%)/100
     ii.    如果无“单价(无税)”和“单价”字段，但有“金额(无税)”和“金额”字段，则设置
     1)    金额(无税)=金额/(1+税率(%)/100)
     2)    如果有“税额”字段，则设置：税额= 金额(无税)* 税率(%)/100
     * @param editTableApi
     * @param fileds
     * @param rowKey
     */
    setNotTaxPriceAndAllMnyByTaxRate: function setNotTaxPriceAndAllMnyByTaxRate(editTableApi, fileds, rowKey) {
        this.calcPrice(editTableApi, fileds, rowKey);
        this.calcMny(editTableApi, fileds, rowKey);
        this.calcMny2(editTableApi, fileds, rowKey);
        this.calcMnyTax(editTableApi, fileds, rowKey);
    },

    /**
     * "数量"变更
     *
     * 编辑表体“数量”字段，设置“税额”、“税额(本币)”、“金额(无税)、“金额(无税)(本币)”和“金额”、“金额(本币)”字段
     * 编辑“数量”字段，会更新设置字段：金额(无税)、金额、税额
     a)    金额(无税)=数量*单价(无税)
     b)    金额=数量*单价
     c)    如果有“税额”字段，则设置：税额=金额(无税)*税率(%)/100
     * @param editTableApi
     * @param fileds
     * @param rowKey
     */
    setAllMnyByNum: function setAllMnyByNum(editTableApi, fileds, rowKey) {
        this.calcMny(editTableApi, fileds, rowKey);
        this.calcTaxMny(editTableApi, fileds, rowKey);
        this.calcMnyTax(editTableApi, fileds, rowKey);
    },

    /**
     * "单价(无税)"变更
     *
     * 编辑“单价(无税)”字段，设置“税额”、“税额(本币)”、“单价”、“金额(无税)、“金额(无税)(本币)”和“金额”、“金额(本币)”字段
     * 编辑“单价(无税)”字段，会更新设置字段：单价、金额(无税)、金额、税额
     a)    单价=单价(无税)*(1+税率(%)/100)
     b)    如果有“金额(无税)”和“金额”字段，则设置
     i.    金额(无税)=单价(无税)*数量
     ii.    金额=单价*数量
     iii.    如果有“税额”字段，则设置：税额=金额(无税)*税率(%)/100
     * @param editTableApi
     * @param fileds
     * @param rowKey
     */
    setAllMnyByPrice: function setAllMnyByPrice(editTableApi, fileds, rowKey) {
        this.calcTaxPrice(editTableApi, fileds, rowKey);
        this.calcMny(editTableApi, fileds, rowKey);
        this.calcTaxMny(editTableApi, fileds, rowKey);
        this.calcMnyTax(editTableApi, fileds, rowKey);
    },

    /**
     * "单价"变更
     *
     * 编辑表头“单价”字段，设置“单价(无税)”、“税额”、“税额(本币)”、“金额(无税)、“金额(无税)(本币)”和“金额”、“金额(本币)”字段
     * 编辑“单价”字段，会更新设置字段：单价(无税)、金额(无税)、金额、税额
     a)    单价(无税)=单价/(1+税率(%)/100)
     b)    如果有“金额(无税)”和“金额”字段，则设置
     i.    金额(无税)=单价(无税)*数量
     ii.    金额=单价*数量
     iii.    如果有“税额”字段，则设置：税额=金额(无税)*税率(%)/1
     * @param editTableApi
     * @param fileds
     * @param rowKey
     */
    setAllMnyByPriceTax: function setAllMnyByPriceTax(editTableApi, fileds, rowKey) {
        this.calcPrice(editTableApi, fileds, rowKey);
        this.calcMny(editTableApi, fileds, rowKey);
        this.calcTaxMny(editTableApi, fileds, rowKey);
        this.calcMnyTax(editTableApi, fileds, rowKey);
    },

    /**
     *"金额(无税)"变更
     * @param editTableApi
     * @param fileds
     * @param rowKey
     */
    setAllMnyByAmountMny: function setAllMnyByAmountMny(editTableApi, fileds, rowKey) {
        if (_priceFirst) {
            // 单价优先
            this.setNumAndAllMnyByAmountMny(editTableApi, fileds, rowKey);
        } else {
            // 数量优先
            this.setPriceAndAllMnyByAmountMny(editTableApi, fileds, rowKey);
        }
    },

    /**
     * * "金额(无税)"变更-单价优先
     *
     * i.    如果有“数量”字段，则“数量”变，“金额”和“税额”不变，只设置：数量=金额(无税)/ 单价(无税)
     ii.    如果没有“数量”字段，则“金额”和“税额”变，只则设置：
     1)    金额=金额(无税)* (1+税率(%)/100)
     2)    如果有“税额”字段，则设置：税额=金额(无税)* (税率(%)/100)
     * @param editTableApi
     * @param fileds
     * @param rowKey
     */
    setNumAndAllMnyByAmountMny: function setNumAndAllMnyByAmountMny(editTableApi, fileds, rowKey) {
        this.calcNum2(editTableApi, fileds, rowKey);
        this.calcTaxMny2(editTableApi, fileds, rowKey);
        this.calcMnyTax(editTableApi, fileds, rowKey);
    },

    /**
     * * "金额(无税)"变更-数量优先
     *
     * i.    如果有“单价(无税)”和“单价”字段，则设置：
     1)    单价(无税)=金额(无税)/ 数量
     2)    单价=单价(无税)* (1+税率(%)/100)
     ii.    金额=金额(无税)* (1+税率(%)/100)
     iii.    如果有“税额”字段，则设置：税额=金额(无税)* (税率(%)/100)
     * @param editTableApi
     * @param fileds
     * @param rowKey
     */
    setPriceAndAllMnyByAmountMny: function setPriceAndAllMnyByAmountMny(editTableApi, fileds, rowKey) {
        this.calcPrice2(editTableApi, fileds, rowKey);
        this.calcTaxPrice(editTableApi, fileds, rowKey);
        this.calcTaxMny2(editTableApi, fileds, rowKey);
        this.calcMnyTax(editTableApi, fileds, rowKey);
    },

    /**
     * "金额"变更
     * @param editTableApi
     * @param fileds
     * @param rowKey
     */
    setAllMnyByAmountTaxMny: function setAllMnyByAmountTaxMny(editTableApi, fileds, rowKey) {
        if (_priceFirst) {
            // 单价优先
            this.setNumAndAllMnyByAmountTaxMny(editTableApi, fileds, rowKey);
        } else {
            // 数量优先
            this.setPriceAndAllMnyByAmountTaxMny(editTableApi, fileds, rowKey);
        }
    },

    /**
     * "金额"变更-单价优先
     *
     * i.    如果有“数量”字段，则“数量”变，“金额(无税)”和“税额”不变，只设置：数量=金额/ 单价
     ii.    如果没有“数量”字段，则“金额(无税)” 和“税额”变，只则设置：
     3)    金额(无税)=金额 / (1 + 税率(%) / 100)
     4)    如果有“税额”字段，则设置：税额=金额(无税)* (税率(%)/100)
     */
    setNumAndAllMnyByAmountTaxMny: function setNumAndAllMnyByAmountTaxMny(editTableApi, fileds, rowKey) {
        this.calcNum(editTableApi, fileds, rowKey);
        this.calcMny2(editTableApi, fileds, rowKey);
        this.calcMnyTax(editTableApi, fileds, rowKey);
    },

    /**
     * "金额"变更-数量优先
     *
     * i.    如果有“单价(无税)”和“单价”字段，则设置：
     5)    单价=金额/ 数量
     6)    单价(无税)=单价/ (1+税率(%)/100)
     ii.    金额(无税)=金额/ (1+税率(%)/100)
     iii.    如果有“税额”字段，则设置：税额=金额(无税)* (税率(%)/100)
     */
    setPriceAndAllMnyByAmountTaxMny: function setPriceAndAllMnyByAmountTaxMny(editTableApi, fileds, rowKey) {
        this.calcTaxPrice2(editTableApi, fileds, rowKey);
        this.calcPrice(editTableApi, fileds, rowKey);
        this.calcMny2(editTableApi, fileds, rowKey);
        this.calcMnyTax(editTableApi, fileds, rowKey);
    },

    /**
     * "税额"变更
     *
     * a)    “金额(无税)”和“金额”为空，则设置：
     i.    金额(无税) = 税额 / (税率(%)/100)
     ii.    金额 = 金额(无税) + 税额
     b)    “金额”为空，“金额(无税)”不为空，则设置：
     i.    金额 = 金额(无税) + 税额
     c)    “金额”不为空，“金额(无税)”为空，则设置：
     i.    金额(无税) = 金额 - 税额
     d)    “金额”和“金额(无税)”均不为空，则根据含税无税优先进行设置：
     ii.    含税优先：金额(无税)=金额-税额
     iii.    无税优先：金额=金额(无税)+税额
     */
    setAllMnyByTaxMny: function setAllMnyByTaxMny(editTableApi, fileds, rowKey) {
        var pk_taxrate = fileds.pk_taxrate,
            ntaxrate = fileds.ntaxrate,
            nnum = fileds.nnum,
            nprice = fileds.nprice,
            ntaxprice = fileds.ntaxprice,
            norigmny = fileds.norigmny,
            norigbilltaxmny = fileds.norigbilltaxmny,
            norigtaxmny = fileds.norigtaxmny;

        var _norigmny = norigmny && editTableApi.getCellDataByKey({ rowKey: rowKey, dataIndex: norigmny });
        var _norigbilltaxmny = norigbilltaxmny && editTableApi.getCellDataByKey({
            rowKey: rowKey,
            dataIndex: norigbilltaxmny
        });
        if (norigmny && norigbilltaxmny && !_norigmny && !_norigbilltaxmny) {
            this.calcMny3(editTableApi, fileds, rowKey);
            this.calcTaxMny3(editTableApi, fileds, rowKey);
        } else if (norigmny && norigbilltaxmny && _norigmny && !_norigbilltaxmny) {
            this.calcTaxMny3(editTableApi, fileds, rowKey);
        } else if (norigmny && norigbilltaxmny && !_norigmny && _norigbilltaxmny) {
            this.calcMny4(editTableApi, fileds, rowKey);
        } else {
            if (_taxFirst) {
                // 含税优先
                this.calcMny4(editTableApi, fileds, rowKey);
                if (_priceFirst) {
                    // 单价优先
                    // 数量 = 金额 / 单价
                    this.calcNum(editTableApi, fileds, rowKey);
                } else {
                    // 数量优先
                    // 单价(无税) = 金额(无税) / 数量(无税)
                    this.calcPrice2(editTableApi, fileds, rowKey);
                    // 单价 = 金额 / 数量
                    this.calcTaxPrice2(editTableApi, fileds, rowKey);
                }
            } else {
                // 无税优先
                this.calcTaxMny3(editTableApi, fileds, rowKey);
                if (_priceFirst) {
                    // 单价优先
                    // 数量 = 金额(无税) / 单价(无税)
                    this.calcNum2(editTableApi, fileds, rowKey);
                } else {
                    // 数量优先
                    // 单价(无税) = 金额(无税) / 数量(无税)
                    this.calcPrice2(editTableApi, fileds, rowKey);
                    // 单价 = 金额 / 数量
                    this.calcTaxPrice2(editTableApi, fileds, rowKey);
                }
            }
        }
    },

    /**
     * 计算金额(无税) = 金额 - 税额
     * @param editTableApi
     * @param fileds
     * @param rowKey
     */
    calcMny4: function calcMny4(editTableApi, fileds, rowKey) {
        var pk_taxrate = fileds.pk_taxrate,
            ntaxrate = fileds.ntaxrate,
            nnum = fileds.nnum,
            nprice = fileds.nprice,
            ntaxprice = fileds.ntaxprice,
            norigmny = fileds.norigmny,
            norigbilltaxmny = fileds.norigbilltaxmny,
            norigtaxmny = fileds.norigtaxmny;

        var _norigbilltaxmny = norigbilltaxmny && editTableApi.getCellDataByKey({
            rowKey: rowKey,
            dataIndex: norigbilltaxmny
        });
        var _norigtaxmny = norigtaxmny && editTableApi.getCellDataByKey({ rowKey: rowKey, dataIndex: norigtaxmny });
        var _cMny4 = void 0;
        _norigtaxmny = !_norigtaxmny ? 0 : _norigtaxmny;
        _norigtaxmny = !_norigtaxmny ? 0 : _norigtaxmny;
        if (norigmny) {
            _cMny4 = _.round(Number(_norigbilltaxmny) - Number(_norigtaxmny), money);
            editTableApi.saveCellData(rowKey, norigmny, _cMny4);
        }
    },

    /**
     * 计算金额 = 金额(无税) + 税额
     * @param editTableApi
     * @param fileds
     * @param rowKey
     */
    calcTaxMny3: function calcTaxMny3(editTableApi, fileds, rowKey) {
        var pk_taxrate = fileds.pk_taxrate,
            ntaxrate = fileds.ntaxrate,
            nnum = fileds.nnum,
            nprice = fileds.nprice,
            ntaxprice = fileds.ntaxprice,
            norigmny = fileds.norigmny,
            norigbilltaxmny = fileds.norigbilltaxmny,
            norigtaxmny = fileds.norigtaxmny;

        var _norigtaxmny = norigtaxmny && editTableApi.getCellDataByKey({ rowKey: rowKey, dataIndex: norigtaxmny });
        var _norigmny = norigmny && editTableApi.getCellDataByKey({ rowKey: rowKey, dataIndex: norigmny });
        var _cTaxMny3 = void 0;
        _norigtaxmny = !_norigtaxmny ? 0 : _norigtaxmny;
        _norigmny = !_norigmny ? 0 : _norigmny;
        if (norigbilltaxmny) {
            _cTaxMny3 = _.round(Number(_norigmny) + Number(_norigtaxmny), money);
            editTableApi.saveCellData(rowKey, norigbilltaxmny, _cTaxMny3);
        }
    },

    /**
     * 计算金额(无税) = 税额 / (税率(%) / 100)
     * @param editTableApi
     * @param fileds
     * @param rowKey
     */
    calcMny3: function calcMny3(editTableApi, fileds, rowKey) {
        var pk_taxrate = fileds.pk_taxrate,
            ntaxrate = fileds.ntaxrate,
            nnum = fileds.nnum,
            nprice = fileds.nprice,
            ntaxprice = fileds.ntaxprice,
            norigmny = fileds.norigmny,
            norigbilltaxmny = fileds.norigbilltaxmny,
            norigtaxmny = fileds.norigtaxmny;

        var _norigtaxmny = norigtaxmny && editTableApi.getCellDataByKey({ rowKey: rowKey, dataIndex: norigtaxmny });
        var _ntaxrate = ntaxrate && editTableApi.getCellDataByKey({ rowKey: rowKey, dataIndex: ntaxrate });
        var _cMny3 = void 0;
        _norigtaxmny = !_norigtaxmny ? 0 : _norigtaxmny;
        _ntaxrate = !_ntaxrate ? 0 : _ntaxrate;
        _ntaxrate = Number(_ntaxrate);
        if (norigmny && !!_ntaxrate) {
            _cMny3 = _.round(Number(_norigtaxmny) / (_ntaxrate / 100), money);
            editTableApi.saveCellData(rowKey, norigmny, _cMny3);
        }
    },

    /**
     * 计算单价 = 金额 / 数量
     * @param editTableApi
     * @param fileds
     * @param rowKey
     */
    calcTaxPrice2: function calcTaxPrice2(editTableApi, fileds, rowKey) {
        var pk_taxrate = fileds.pk_taxrate,
            ntaxrate = fileds.ntaxrate,
            nnum = fileds.nnum,
            nprice = fileds.nprice,
            ntaxprice = fileds.ntaxprice,
            norigmny = fileds.norigmny,
            norigbilltaxmny = fileds.norigbilltaxmny,
            norigtaxmny = fileds.norigtaxmny;

        var _norigbilltaxmny = editTableApi.getCellDataByKey({ rowKey: rowKey, dataIndex: norigbilltaxmny });
        var _nnum = editTableApi.getCellDataByKey({ rowKey: rowKey, dataIndex: nnum });
        var _cTaxPrice2 = void 0;
        _norigbilltaxmny = !_norigbilltaxmny ? 0 : _norigbilltaxmny;
        _nnum = !_nnum ? 0 : _nnum;
        _nnum = Number(_nnum);
        if (ntaxprice && !!_nnum) {
            _cTaxPrice2 = _.round(Number(_norigbilltaxmny) / _nnum, price);
            editTableApi.saveCellData(rowKey, ntaxprice, _cTaxPrice2);
        }
    },

    /**
     * 计算数量 = 金额 / 单价
     * @param editTableApi
     * @param fileds
     * @param rowKey
     */
    calcNum: function calcNum(editTableApi, fileds, rowKey) {
        var pk_taxrate = fileds.pk_taxrate,
            ntaxrate = fileds.ntaxrate,
            nnum = fileds.nnum,
            nprice = fileds.nprice,
            ntaxprice = fileds.ntaxprice,
            norigmny = fileds.norigmny,
            norigbilltaxmny = fileds.norigbilltaxmny,
            norigtaxmny = fileds.norigtaxmny;

        var _norigbilltaxmny = norigbilltaxmny && editTableApi.getCellDataByKey({
            rowKey: rowKey,
            dataIndex: norigbilltaxmny
        });
        var _ntaxprice = ntaxprice && editTableApi.getCellDataByKey({ rowKey: rowKey, dataIndex: ntaxprice });
        var _cNum = void 0;
        _norigbilltaxmny = !_norigbilltaxmny ? 0 : _norigbilltaxmny;
        _ntaxprice = !_ntaxprice ? 0 : _ntaxprice;
        _ntaxprice = Number(_ntaxprice);
        if (nnum && !!_ntaxprice) {
            _cNum = _.round(Number(_norigbilltaxmny) / _ntaxprice, num);
            editTableApi.saveCellData(rowKey, nnum, _cNum);
        }
    },

    /**
     * 计算单价无税 = 金额(无税) / 数量
     * @param editTableApi
     * @param fileds
     * @param rowKey
     */
    calcPrice2: function calcPrice2(editTableApi, fileds, rowKey) {
        var pk_taxrate = fileds.pk_taxrate,
            ntaxrate = fileds.ntaxrate,
            nnum = fileds.nnum,
            nprice = fileds.nprice,
            ntaxprice = fileds.ntaxprice,
            norigmny = fileds.norigmny,
            norigbilltaxmny = fileds.norigbilltaxmny,
            norigtaxmny = fileds.norigtaxmny;

        var _norigmny = norigmny && editTableApi.getCellDataByKey({ rowKey: rowKey, dataIndex: norigmny });
        var _nnum = nnum && editTableApi.getCellDataByKey({ rowKey: rowKey, dataIndex: nnum });
        var _cPrice2 = void 0;
        _norigmny = !_norigmny ? 0 : _norigmny;
        _nnum = !_nnum ? 0 : _nnum;
        _nnum = Number(_nnum);
        if (nprice && !!_nnum) {
            _cPrice2 = _.round(Number(_norigmny) / _nnum, price);
            editTableApi.saveCellData(rowKey, nprice, _cPrice2);
        }
    },

    /**
     * 计算数量 = 金额(无税) / 单价(无税)
     * @param editTableApi
     * @param fileds
     * @param rowKey
     */
    calcNum2: function calcNum2(editTableApi, fileds, rowKey) {
        var pk_taxrate = fileds.pk_taxrate,
            ntaxrate = fileds.ntaxrate,
            nnum = fileds.nnum,
            nprice = fileds.nprice,
            ntaxprice = fileds.ntaxprice,
            norigmny = fileds.norigmny,
            norigbilltaxmny = fileds.norigbilltaxmny,
            norigtaxmny = fileds.norigtaxmny;

        var _norigmny = norigmny && editTableApi.getCellDataByKey({ rowKey: rowKey, dataIndex: norigmny });
        var _nprice = nprice && editTableApi.getCellDataByKey({ rowKey: rowKey, dataIndex: nprice });
        var _cNum2 = void 0;
        _norigmny = !_norigmny ? 0 : _norigmny;
        _nprice = !_nprice ? 0 : _nprice;
        _nprice = Number(_nprice);
        if (nnum && !!_nprice) {
            _cNum2 = _.round(Number(_norigmny) / _nprice, num);
            editTableApi.saveCellData(rowKey, nnum, _cNum2);
        }
    },

    /**
     * 计算金额 = 金额(无税) * (1 + 税率(%) / 100)
     * @param editTableApi
     * @param fileds
     * @param rowKey
     * @returns {*}
     */
    calcTaxMny2: function calcTaxMny2(editTableApi, fileds, rowKey) {
        var pk_taxrate = fileds.pk_taxrate,
            ntaxrate = fileds.ntaxrate,
            nnum = fileds.nnum,
            nprice = fileds.nprice,
            ntaxprice = fileds.ntaxprice,
            norigmny = fileds.norigmny,
            norigbilltaxmny = fileds.norigbilltaxmny,
            norigtaxmny = fileds.norigtaxmny;

        var _ntaxrate = ntaxrate && editTableApi.getCellDataByKey({ rowKey: rowKey, dataIndex: ntaxrate });
        var _norigmny = norigmny && editTableApi.getCellDataByKey({ rowKey: rowKey, dataIndex: norigmny });
        var _cTaxMny2 = void 0;
        if (!_ntaxrate) {
            _ntaxrate = 0;
        }
        _norigmny = !_norigmny ? 0 : _norigmny;
        if (norigbilltaxmny) {
            _cTaxMny2 = _.round(Number(_norigmny) * (1 + Number(_ntaxrate) / 100), money);
            editTableApi.saveCellData(rowKey, norigbilltaxmny, _cTaxMny2);
        }
    },

    /**
     * 计算金额 = 单价 * 数量
     * @param editTableApi
     * @param fileds
     * @param rowKey
     * @returns {*}
     */
    calcTaxMny: function calcTaxMny(editTableApi, fileds, rowKey) {
        var pk_taxrate = fileds.pk_taxrate,
            ntaxrate = fileds.ntaxrate,
            nnum = fileds.nnum,
            nprice = fileds.nprice,
            ntaxprice = fileds.ntaxprice,
            norigmny = fileds.norigmny,
            norigbilltaxmny = fileds.norigbilltaxmny,
            norigtaxmny = fileds.norigtaxmny;

        var _ntaxprice = ntaxprice && editTableApi.getCellDataByKey({ rowKey: rowKey, dataIndex: ntaxprice });
        var _nnum = nnum && editTableApi.getCellDataByKey({ rowKey: rowKey, dataIndex: nnum });
        var _cTaxMny = void 0;
        _ntaxprice = !_ntaxprice ? 0 : _ntaxprice;
        _nnum = !_nnum ? 0 : _nnum;
        if (norigbilltaxmny) {
            _cTaxMny = _.round(Number(_ntaxprice) * Number(_nnum), money);
            editTableApi.saveCellData(rowKey, norigbilltaxmny, _cTaxMny);
        }
    },

    /**
     * 计算单价 = 单价(无税) * (1 + 税率(%) / 100)
     * @param editTableApi
     * @param fileds
     * @param rowKey
     * @returns {*}
     */
    calcTaxPrice: function calcTaxPrice(editTableApi, fileds, rowKey) {
        var pk_taxrate = fileds.pk_taxrate,
            ntaxrate = fileds.ntaxrate,
            nnum = fileds.nnum,
            nprice = fileds.nprice,
            ntaxprice = fileds.ntaxprice,
            norigmny = fileds.norigmny,
            norigbilltaxmny = fileds.norigbilltaxmny,
            norigtaxmny = fileds.norigtaxmny;

        var _nprice = nprice && editTableApi.getCellDataByKey({ rowKey: rowKey, dataIndex: nprice });
        var _ntaxrate = ntaxrate && editTableApi.getCellDataByKey({ rowKey: rowKey, dataIndex: ntaxrate });
        var _cTaxPrice = void 0;
        if (!_ntaxrate) {
            _ntaxrate = 0;
        }
        _nprice = !_nprice ? 0 : _nprice;
        if (ntaxprice) {
            _cTaxPrice = _.round(Number(_nprice) * (1 + Number(_ntaxrate) / 100), price);
            editTableApi.saveCellData(rowKey, ntaxprice, _cTaxPrice);
        }
    },

    /**
     * 计算单价无税 = 单价 / (1 + 税率(%) / 100)
     * @param editTableApi
     * @param fileds
     * @param rowKey
     */
    calcPrice: function calcPrice(editTableApi, fileds, rowKey) {
        var pk_taxrate = fileds.pk_taxrate,
            ntaxrate = fileds.ntaxrate,
            nnum = fileds.nnum,
            nprice = fileds.nprice,
            ntaxprice = fileds.ntaxprice,
            norigmny = fileds.norigmny,
            norigbilltaxmny = fileds.norigbilltaxmny,
            norigtaxmny = fileds.norigtaxmny;

        var _ntaxprice = ntaxprice && editTableApi.getCellDataByKey({ rowKey: rowKey, dataIndex: ntaxprice });
        var _ntaxrate = ntaxrate && editTableApi.getCellDataByKey({ rowKey: rowKey, dataIndex: ntaxrate });
        var _cPrice = void 0;
        if (!_ntaxrate) {
            _ntaxrate = 0;
        }
        _ntaxprice = !_ntaxprice ? 0 : _ntaxprice;
        if (nprice) {
            _cPrice = _.round(Number(_ntaxprice) / (1 + Number(_ntaxrate) / 100), price);
            editTableApi.saveCellData(rowKey, nprice, _cPrice);
        }
    },

    /**
     *计算金额(无税) = 单价(无税) * 数量
     * @param editTableApi
     * @param fileds
     * @param rowKey
     */
    calcMny: function calcMny(editTableApi, fileds, rowKey) {
        var pk_taxrate = fileds.pk_taxrate,
            ntaxrate = fileds.ntaxrate,
            nnum = fileds.nnum,
            nprice = fileds.nprice,
            ntaxprice = fileds.ntaxprice,
            norigmny = fileds.norigmny,
            norigbilltaxmny = fileds.norigbilltaxmny,
            norigtaxmny = fileds.norigtaxmny;

        var _nprice = nprice && editTableApi.getCellDataByKey({ rowKey: rowKey, dataIndex: nprice });
        var _nnum = nnum && editTableApi.getCellDataByKey({ rowKey: rowKey, dataIndex: nnum });
        var _cMny = void 0;
        _nprice = !_nprice ? 0 : _nprice;
        _nnum = !_nnum ? 0 : _nnum;
        if (norigmny) {
            _cMny = _.round(Number(_nprice) * Number(_nnum), money);
            editTableApi.saveCellData(rowKey, norigmny, _cMny);
        }
    },

    /**
     *计算金额(无税) = 金额 / (1 + 税率(%) / 100)
     * @param editTableApi
     * @param fileds
     * @param rowKey
     */
    calcMny2: function calcMny2(editTableApi, fileds, rowKey) {
        var pk_taxrate = fileds.pk_taxrate,
            ntaxrate = fileds.ntaxrate,
            nnum = fileds.nnum,
            nprice = fileds.nprice,
            ntaxprice = fileds.ntaxprice,
            norigmny = fileds.norigmny,
            norigbilltaxmny = fileds.norigbilltaxmny,
            norigtaxmny = fileds.norigtaxmny;

        var _norigbilltaxmny = norigbilltaxmny && editTableApi.getCellDataByKey({
            rowKey: rowKey,
            dataIndex: norigbilltaxmny
        });
        var _ntaxrate = ntaxrate && editTableApi.getCellDataByKey({ rowKey: rowKey, dataIndex: ntaxrate });
        var _cMny2 = void 0;
        if (!_ntaxrate) {
            _ntaxrate = 0;
        }
        _norigbilltaxmny = !_norigbilltaxmny ? 0 : _norigbilltaxmny;
        if (norigmny) {
            _cMny2 = _.round(Number(_norigbilltaxmny) / (1 + Number(_ntaxrate) / 100), money);
            editTableApi.saveCellData(rowKey, norigmny, _cMny2);
        }
    },

    /**
     * 计算税额 = 金额(无税) * 税率(%) / 100
     * @param editTableApi
     * @param fileds
     * @param rowKey
     * @returns {*}
     */
    calcMnyTax: function calcMnyTax(editTableApi, fileds, rowKey) {
        var pk_taxrate = fileds.pk_taxrate,
            ntaxrate = fileds.ntaxrate,
            nnum = fileds.nnum,
            nprice = fileds.nprice,
            ntaxprice = fileds.ntaxprice,
            norigmny = fileds.norigmny,
            norigbilltaxmny = fileds.norigbilltaxmny,
            norigtaxmny = fileds.norigtaxmny;

        var _norigmny = norigmny && editTableApi.getCellDataByKey({ rowKey: rowKey, dataIndex: norigmny });
        var _ntaxrate = ntaxrate && editTableApi.getCellDataByKey({ rowKey: rowKey, dataIndex: ntaxrate });
        var _cMnyTax = void 0;
        if (!_ntaxrate) {
            _ntaxrate = 0;
        }
        _norigmny = !_norigmny ? 0 : _norigmny;
        if (norigtaxmny) {
            _cMnyTax = _.round(Number(_norigmny) * Number(_ntaxrate) / 100, rate);
            editTableApi.saveCellData(rowKey, norigtaxmny, _cMnyTax);
        }
    },

    /**
     * * "税率"变更
     * 编辑表头"税率"字段，根据参照“含税优先/无税优先”，设置单价和金额字段；
     * 如果“含税优先”，设置“单价(无税)”和“税额”、“税额(本币)”、“金额(无税)、“金额(无税)(本币)”字段值；
     * 如果“无税优先”，设置“单价”和“税额”、“税额(本币)”、“金额”、“金额(本币)”字段值
     * @param editFormApi
     * @param fileds
     */
    setHeadAllMnyByTaxRate: function setHeadAllMnyByTaxRate(editFormApi, fileds) {
        if (_taxFirst) {
            // 含税优先
            this.setHeadNotTaxPriceAndAllMnyByTaxRate(editFormApi, fileds);
        } else {
            // 无税优先
            this.setHeadHasTaxPriceAndAllMnyByTaxRate(editFormApi, fileds);
        }
    },

    /**
     * * "税率"变更-无税优先
     *
     * 编辑表头"税率"字段，设置“单价”和“税额”、“税额(本币)”、“金额”、“金额(本币)”字段值
     * 1)    如果有“单价(无税)”和“单价”字段，则设置
     单价=单价(无税)*(1+税率(%)/100)
     2)    如果有“金额(无税)”和“金额”，则设置
     金额= 单价*数量
     3)    如果有“税额”字段，则设置：
     税额= 金额(无税)* 税率(%)/100
     ii.    如果无“单价(无税)”和“单价”字段，但有“金额(无税)”和“金额”字段，则设置
     1)    金额=金额(无税)*(1+税率(%)/100)
     2)    如果有“税额”字段，则设置：税额= 金额(无税)* 税率(%)/100
     * @param editFormApi
     * @param fileds
     */
    setHeadHasTaxPriceAndAllMnyByTaxRate: function setHeadHasTaxPriceAndAllMnyByTaxRate(editFormApi, fileds) {
        this.calcHeadTaxPrice(editFormApi, fileds);
        this.calcHeadTaxMny(editFormApi, fileds);
        this.calcHeadTaxMny2(editFormApi, fileds);
        this.calcHeadMnyTax(editFormApi, fileds);
    },

    /**
     * * "税率"变更-含税优先
     *
     * 编辑表头"税率"字段，设置“单价(无税)”和“税额”、“税额(本币)”、“金额(无税)、“金额(无税)(本币)”字段值
     * 1)    如果有“单价(无税)”和“单价”字段，则设置
     单价(无税)=单价/(1+税率(%)/100)
     2)    如果有“金额(无税)”和“金额”，则设置
     金额(无税)= 单价(无税)*数量
     3)    如果有“税额”字段，则设置：
     税额= 金额(无税)* 税率(%)/100
     ii.    如果无“单价(无税)”和“单价”字段，但有“金额(无税)”和“金额”字段，则设置
     1)    金额(无税)=金额/(1+税率(%)/100)
     2)    如果有“税额”字段，则设置：税额= 金额(无税)* 税率(%)/100
     * @param editFormApi
     * @param fileds
     */
    setHeadNotTaxPriceAndAllMnyByTaxRate: function setHeadNotTaxPriceAndAllMnyByTaxRate(editFormApi, fileds) {
        this.calcHeadPrice(editFormApi, fileds);
        this.calcHeadMny(editFormApi, fileds);
        this.calcHeadMny2(editFormApi, fileds);
        this.calcHeadMnyTax(editFormApi, fileds);
    },

    /**
     * * "数量"变更
     *
     * 编辑表体“数量”字段，设置“税额”、“税额(本币)”、“金额(无税)、“金额(无税)(本币)”和“金额”、“金额(本币)”字段
     * 编辑“数量”字段，会更新设置字段：金额(无税)、金额、税额
     a)    金额(无税)=数量*单价(无税)
     b)    金额=数量*单价
     c)    如果有“税额”字段，则设置：税额=金额(无税)*税率(%)/100
     * @param editFormApi
     * @param fileds
     */
    setHeadAllMnyByNum: function setHeadAllMnyByNum(editFormApi, fileds) {
        this.calcHeadMny(editFormApi, fileds);
        this.calcHeadTaxMny(editFormApi, fileds);
        this.calcHeadMnyTax(editFormApi, fileds);
    },

    /**
     * * "单价(无税)"变更
     *
     * 编辑“单价(无税)”字段，设置“税额”、“税额(本币)”、“单价”、“金额(无税)、“金额(无税)(本币)”和“金额”、“金额(本币)”字段
     * 编辑“单价(无税)”字段，会更新设置字段：单价、金额(无税)、金额、税额
     a)    单价=单价(无税)*(1+税率(%)/100)
     b)    如果有“金额(无税)”和“金额”字段，则设置
     i.    金额(无税)=单价(无税)*数量
     ii.    金额=单价*数量
     iii.    如果有“税额”字段，则设置：税额=金额(无税)*税率(%)/100
     * @param editFormApi
     * @param fileds
     */
    setHeadAllMnyByPrice: function setHeadAllMnyByPrice(editFormApi, fileds) {
        this.calcHeadTaxPrice(editFormApi, fileds);
        this.calcHeadMny(editFormApi, fileds);
        this.calcHeadTaxMny(editFormApi, fileds);
        this.calcHeadMnyTax(editFormApi, fileds);
    },

    /**
     * * "单价"变更
     *
     * 编辑表头“单价”字段，设置“单价(无税)”、“税额”、“税额(本币)”、“金额(无税)、“金额(无税)(本币)”和“金额”、“金额(本币)”字段
     * 编辑“单价”字段，会更新设置字段：单价(无税)、金额(无税)、金额、税额
     a)    单价(无税)=单价/(1+税率(%)/100)
     b)    如果有“金额(无税)”和“金额”字段，则设置
     i.    金额(无税)=单价(无税)*数量
     ii.    金额=单价*数量
     iii.    如果有“税额”字段，则设置：税额=金额(无税)*税率(%)/1
     * @param editFormApi
     * @param fileds
     */
    setHeadAllMnyByPriceTax: function setHeadAllMnyByPriceTax(editFormApi, fileds) {
        this.calcHeadPrice(editFormApi, fileds);
        this.calcHeadMny(editFormApi, fileds);
        this.calcHeadTaxMny(editFormApi, fileds);
        this.calcHeadMnyTax(editFormApi, fileds);
    },

    /**
     * "金额(无税)"变更
     * @param editFormApi
     * @param fileds
     */
    setHeadAllMnyByAmountMny: function setHeadAllMnyByAmountMny(editFormApi, fileds) {
        if (_priceFirst) {
            // 单价优先
            this.setHeadNumAndAllMnyByAmountMny(editFormApi, fileds);
        } else {
            // 数量优先
            this.setHeadPriceAndAllMnyByAmountMny(editFormApi, fileds);
        }
    },

    /**
     * * * "金额(无税)"变更-单价优先
     *
     * i.    如果有“数量”字段，则“数量”变，“金额”和“税额”不变，只设置：数量=金额(无税)/ 单价(无税)
     ii.    如果没有“数量”字段，则“金额”和“税额”变，只则设置：
     1)    金额=金额(无税)* (1+税率(%)/100)
     2)    如果有“税额”字段，则设置：税额=金额(无税)* (税率(%)/100)
     * @param editFormApi
     * @param fileds
     */
    setHeadNumAndAllMnyByAmountMny: function setHeadNumAndAllMnyByAmountMny(editFormApi, fileds) {
        this.calcHeadNum2(editFormApi, fileds);
        this.calcHeadTaxMny2(editFormApi, fileds);
        this.calcHeadMnyTax(editFormApi, fileds);
    },

    /**
     * * * "金额(无税)"变更-数量优先
     *
     * i.    如果有“单价(无税)”和“单价”字段，则设置：
     1)    单价(无税)=金额(无税)/ 数量
     2)    单价=单价(无税)* (1+税率(%)/100)
     ii.    金额=金额(无税)* (1+税率(%)/100)
     iii.    如果有“税额”字段，则设置：税额=金额(无税)* (税率(%)/100)
     * @param editFormApi
     * @param fileds
     */
    setHeadPriceAndAllMnyByAmountMny: function setHeadPriceAndAllMnyByAmountMny(editFormApi, fileds) {
        this.calcHeadPrice2(editFormApi, fileds);
        this.calcHeadTaxPrice(editFormApi, fileds);
        this.calcHeadTaxMny2(editFormApi, fileds);
        this.calcHeadMnyTax(editFormApi, fileds);
    },

    /**
     * "金额"变更
     * @param editFormApi
     * @param fileds
     */
    setHeadAllMnyByAmountTaxMny: function setHeadAllMnyByAmountTaxMny(editFormApi, fileds) {
        if (_priceFirst) {
            // 单价优先
            this.setHeadNumAndAllMnyByAmountTaxMny(editFormApi, fileds);
        } else {
            // 数量优先
            this.setHeadPriceAndAllMnyByAmountTaxMny(editFormApi, fileds);
        }
    },

    /**
     * * "金额"变更-单价优先
     *
     * i.    如果有“数量”字段，则“数量”变，“金额(无税)”和“税额”不变，只设置：数量=金额/ 单价
     ii.    如果没有“数量”字段，则“金额(无税)” 和“税额”变，只则设置：
     3)    金额(无税)=金额 / (1 + 税率(%) / 100)
     4)    如果有“税额”字段，则设置：税额=金额(无税)* (税率(%)/100)
     * @param editFormApi
     * @param fileds
     */
    setHeadNumAndAllMnyByAmountTaxMny: function setHeadNumAndAllMnyByAmountTaxMny(editFormApi, fileds) {
        this.calcHeadNum(editFormApi, fileds);
        this.calcHeadMny2(editFormApi, fileds);
        this.calcHeadMnyTax(editFormApi, fileds);
    },

    /**
     *  * "金额"变更-数量优先
     *
     * i.    如果有“单价(无税)”和“单价”字段，则设置：
     5)    单价=金额/ 数量
     6)    单价(无税)=单价/ (1+税率(%)/100)
     ii.    金额(无税)=金额/ (1+税率(%)/100)
     iii.    如果有“税额”字段，则设置：税额=金额(无税)* (税率(%)/100)
     * @param editFormApi
     * @param fileds
     */
    setHeadPriceAndAllMnyByAmountTaxMny: function setHeadPriceAndAllMnyByAmountTaxMny(editFormApi, fileds) {
        this.calcHeadTaxPrice2(editFormApi, fileds);
        this.calcHeadPrice(editFormApi, fileds);
        this.calcHeadMny2(editFormApi, fileds);
        this.calcHeadMnyTax(editFormApi, fileds);
    },

    /**
     * "税额"变更
     *
     * a)    “金额(无税)”和“金额”为空，则设置：
     i.    金额(无税) = 税额 / (税率(%)/100)
     ii.    金额 = 金额(无税) + 税额
     b)    “金额”为空，“金额(无税)”不为空，则设置：
     i.    金额 = 金额(无税) + 税额
     c)    “金额”不为空，“金额(无税)”为空，则设置：
     i.    金额(无税) = 金额 - 税额
     d)    “金额”和“金额(无税)”均不为空，则根据含税无税优先进行设置：
     ii.    含税优先：金额(无税)=金额-税额
     iii.    无税优先：金额=金额(无税)+税额
     */
    setHeadAllMnyByTaxMny: function setHeadAllMnyByTaxMny(editFormApi, fileds) {
        var pk_taxrate = fileds.pk_taxrate,
            ntaxrate = fileds.ntaxrate,
            nnum = fileds.nnum,
            nprice = fileds.nprice,
            ntaxprice = fileds.ntaxprice,
            norigmny = fileds.norigmny,
            norigbilltaxmny = fileds.norigbilltaxmny,
            norigtaxmny = fileds.norigtaxmny;

        var _norigmny = norigmny && editFormApi.getFieldValue(norigmny);
        var _norigbilltaxmny = norigbilltaxmny && editFormApi.getFieldValue(norigbilltaxmny);
        if (norigmny && norigbilltaxmny && !_norigmny && !_norigbilltaxmny) {
            this.calcHeadMny3(editFormApi, fileds);
            this.calcHeadTaxMny3(editFormApi, fileds);
        } else if (norigmny && norigbilltaxmny && _norigmny && !_norigbilltaxmny) {
            this.calcHeadTaxMny3(editFormApi, fileds);
        } else if (norigmny && norigbilltaxmny && !_norigmny && _norigbilltaxmny) {
            this.calcHeadMny4(editFormApi, fileds);
        } else {
            if (_taxFirst) {
                // 含税优先
                this.calcHeadMny4(editFormApi, fileds);
                if (_priceFirst) {
                    // 单价优先
                    // 数量 = 金额 / 单价
                    this.calcHeadNum(editFormApi, fileds);
                } else {
                    // 数量优先
                    // 单价(无税) = 金额(无税) / 数量(无税)
                    this.calcHeadPrice2(editFormApi, fileds);
                    // 单价 = 金额 / 数量
                    this.calcHeadTaxPrice2(editFormApi, fileds);
                }
            } else {
                // 无税优先
                this.calcHeadTaxMny3(editFormApi, fileds);
                if (_priceFirst) {
                    // 单价优先
                    // 数量 = 金额(无税) / 单价(无税)
                    this.calcHeadNum2(editFormApi, fileds);
                } else {
                    // 数量优先
                    // 单价(无税) = 金额(无税) / 数量(无税)
                    this.calcHeadPrice2(editFormApi, fileds);
                    // 单价 = 金额 / 数量
                    this.calcHeadTaxPrice2(editFormApi, fileds);
                }
            }
        }
    },

    /**
     * 计算金额(无税) = 金额 - 税额
     * @param editFormApi
     * @param fileds
     */
    calcHeadMny4: function calcHeadMny4(editFormApi, fileds) {
        var pk_taxrate = fileds.pk_taxrate,
            ntaxrate = fileds.ntaxrate,
            nnum = fileds.nnum,
            nprice = fileds.nprice,
            ntaxprice = fileds.ntaxprice,
            norigmny = fileds.norigmny,
            norigbilltaxmny = fileds.norigbilltaxmny,
            norigtaxmny = fileds.norigtaxmny;

        var _norigbilltaxmny = norigbilltaxmny && editFormApi.getFieldValue(norigbilltaxmny);
        var _norigtaxmny = norigtaxmny && editFormApi.getFieldValue(norigtaxmny);
        var _cMny4 = void 0;
        _norigbilltaxmny = !_norigbilltaxmny ? 0 : _norigbilltaxmny;
        _norigtaxmny = !_norigtaxmny ? 0 : _norigtaxmny;
        if (norigmny) {
            _cMny4 = Number(_norigbilltaxmny) - Number(_norigtaxmny);
            var _data = {};
            _data[norigmny] = _cMny4 ? _.round(_cMny4, money) : _.round(0, money);
            editFormApi.setFieldsValue(_data);
        }
    },

    /**
     * 计算金额 = 金额(无税) + 税额
     * @param editFormApi
     * @param fileds
     */
    calcHeadTaxMny3: function calcHeadTaxMny3(editFormApi, fileds) {
        var pk_taxrate = fileds.pk_taxrate,
            ntaxrate = fileds.ntaxrate,
            nnum = fileds.nnum,
            nprice = fileds.nprice,
            ntaxprice = fileds.ntaxprice,
            norigmny = fileds.norigmny,
            norigbilltaxmny = fileds.norigbilltaxmny,
            norigtaxmny = fileds.norigtaxmny;

        var _norigtaxmny = norigtaxmny && editFormApi.getFieldValue(norigtaxmny);
        var _norigmny = norigmny && editFormApi.getFieldValue(norigmny);
        var _cTaxMny3 = void 0;
        _norigtaxmny = !_norigtaxmny ? 0 : _norigtaxmny;
        _norigmny = !_norigmny ? 0 : _norigmny;
        if (norigbilltaxmny) {
            _cTaxMny3 = Number(_norigmny) + Number(_norigtaxmny);
            var _data = {};
            _data[norigbilltaxmny] = _cTaxMny3 ? _.round(_cTaxMny3, money) : _.round(0, money);
            editFormApi.setFieldsValue(_data);
        }
    },

    /**
     * 计算金额(无税) = 税额 / (税率(%) / 100)
     * @param editFormApi
     * @param fileds
     */
    calcHeadMny3: function calcHeadMny3(editFormApi, fileds) {
        var pk_taxrate = fileds.pk_taxrate,
            ntaxrate = fileds.ntaxrate,
            nnum = fileds.nnum,
            nprice = fileds.nprice,
            ntaxprice = fileds.ntaxprice,
            norigmny = fileds.norigmny,
            norigbilltaxmny = fileds.norigbilltaxmny,
            norigtaxmny = fileds.norigtaxmny;

        var _norigtaxmny = norigtaxmny && editFormApi.getFieldValue(norigtaxmny);
        var _ntaxrate = ntaxrate && editFormApi.getFieldValue(ntaxrate);
        var _cMny3 = void 0;
        _norigtaxmny = !_norigtaxmny ? 0 : _norigtaxmny;
        _ntaxrate = !_ntaxrate ? 0 : _ntaxrate;
        _ntaxrate = Number(_ntaxrate);
        if (norigmny && !!_ntaxrate) {
            _cMny3 = Number(_norigtaxmny) / (_ntaxrate / 100);
            var _data = {};
            _data[norigmny] = _cMny3 ? _.round(_cMny3, money) : _.round(0, money);
            editFormApi.setFieldsValue(_data);
        }
    },

    /**
     *  计算单价 = 金额 / 数量
     * @param editFormApi
     * @param fileds
     */
    calcHeadTaxPrice2: function calcHeadTaxPrice2(editFormApi, fileds) {
        var pk_taxrate = fileds.pk_taxrate,
            ntaxrate = fileds.ntaxrate,
            nnum = fileds.nnum,
            nprice = fileds.nprice,
            ntaxprice = fileds.ntaxprice,
            norigmny = fileds.norigmny,
            norigbilltaxmny = fileds.norigbilltaxmny,
            norigtaxmny = fileds.norigtaxmny;

        var _norigbilltaxmny = norigbilltaxmny && editFormApi.getFieldValue(norigbilltaxmny);
        var _nnum = nnum && editFormApi.getFieldValue(nnum);
        var _cTaxPrice2 = void 0;
        _norigbilltaxmny = !_norigbilltaxmny ? 0 : _norigbilltaxmny;
        _nnum = !_nnum ? 0 : _nnum;
        _nnum = Number(_nnum);
        if (ntaxprice && !!_nnum) {
            _cTaxPrice2 = Number(_norigbilltaxmny) / _nnum;
            var _data = {};
            _data[ntaxprice] = _cTaxPrice2 ? _.round(_cTaxPrice2, price) : _.round(0, price);
            editFormApi.setFieldsValue(_data);
        }
    },

    /**
     * 计算数量 = 金额 / 单价
     * @param editFormApi
     * @param fileds
     */
    calcHeadNum: function calcHeadNum(editFormApi, fileds) {
        var pk_taxrate = fileds.pk_taxrate,
            ntaxrate = fileds.ntaxrate,
            nnum = fileds.nnum,
            nprice = fileds.nprice,
            ntaxprice = fileds.ntaxprice,
            norigmny = fileds.norigmny,
            norigbilltaxmny = fileds.norigbilltaxmny,
            norigtaxmny = fileds.norigtaxmny;

        var _norigbilltaxmny = norigbilltaxmny && editFormApi.getFieldValue(norigbilltaxmny);
        var _ntaxprice = ntaxprice && editFormApi.getFieldValue(ntaxprice);
        var _cNum = void 0;
        _norigbilltaxmny = !_norigbilltaxmny ? 0 : _norigbilltaxmny;
        _ntaxprice = !_ntaxprice ? 0 : _ntaxprice;
        _ntaxprice = Number(_ntaxprice);
        if (nnum && !!_ntaxprice) {
            _cNum = Number(_norigbilltaxmny) / _ntaxprice;
            var _data = {};
            _data[nnum] = _cNum ? _.round(_cNum, num) : _.round(0, num);
            editFormApi.setFieldsValue(_data);
        }
    },

    /**
     * 计算单价无税 = 金额(无税) / 数量
     * @param editFormApi
     * @param fileds
     */
    calcHeadPrice2: function calcHeadPrice2(editFormApi, fileds) {
        var pk_taxrate = fileds.pk_taxrate,
            ntaxrate = fileds.ntaxrate,
            nnum = fileds.nnum,
            nprice = fileds.nprice,
            ntaxprice = fileds.ntaxprice,
            norigmny = fileds.norigmny,
            norigbilltaxmny = fileds.norigbilltaxmny,
            norigtaxmny = fileds.norigtaxmny;

        var _norigmny = norigmny && editFormApi.getFieldValue(norigmny);
        var _nnum = nnum && editFormApi.getFieldValue(nnum);
        var _cPrice2 = void 0;
        _norigmny = !_norigmny ? 0 : _norigmny;
        _nnum = !_nnum ? 0 : _nnum;
        _nnum = Number(_nnum);
        if (nprice && !!_nnum) {
            _cPrice2 = Number(_norigmny) / _nnum;
            var _data = {};
            _data[nprice] = _cPrice2 ? _.round(_cPrice2, price) : _.round(0, price);
            editFormApi.setFieldsValue(_data);
        }
    },

    /**
     * 计算数量 = 金额(无税) / 单价(无税)
     * @param editFormApi
     * @param fileds
     */
    calcHeadNum2: function calcHeadNum2(editFormApi, fileds) {
        var pk_taxrate = fileds.pk_taxrate,
            ntaxrate = fileds.ntaxrate,
            nnum = fileds.nnum,
            nprice = fileds.nprice,
            ntaxprice = fileds.ntaxprice,
            norigmny = fileds.norigmny,
            norigbilltaxmny = fileds.norigbilltaxmny,
            norigtaxmny = fileds.norigtaxmny;

        var _norigmny = norigmny && editFormApi.getFieldValue(norigmny);
        var _nprice = nprice && editFormApi.getFieldValue(nprice);
        var _cNum2 = void 0;
        _norigmny = !_norigmny ? 0 : _norigmny;
        _nprice = !_nprice ? 0 : _nprice;
        _nprice = Number(_nprice);
        if (nnum && !!_nprice) {
            _cNum2 = Number(_norigmny) / _nprice;
            var _data = {};
            _data[nnum] = _cNum2 ? _.round(_cNum2, num) : _.round(0, num);
            editFormApi.setFieldsValue(_data);
        }
    },

    /**
     * 计算金额 = 金额(无税) * (1 + 税率(%) / 100)
     * @param editFormApi
     * @param fileds
     */
    calcHeadTaxMny2: function calcHeadTaxMny2(editFormApi, fileds) {
        var pk_taxrate = fileds.pk_taxrate,
            ntaxrate = fileds.ntaxrate,
            nnum = fileds.nnum,
            nprice = fileds.nprice,
            ntaxprice = fileds.ntaxprice,
            norigmny = fileds.norigmny,
            norigbilltaxmny = fileds.norigbilltaxmny,
            norigtaxmny = fileds.norigtaxmny;

        var _ntaxrate = ntaxrate && editFormApi.getFieldValue(ntaxrate);
        var _norigmny = norigmny && editFormApi.getFieldValue(norigmny);
        var _cTaxMny2 = void 0;
        if (!_ntaxrate) {
            _ntaxrate = 0;
        }
        _norigmny = !_norigmny ? 0 : _norigmny;
        if (norigbilltaxmny) {
            _cTaxMny2 = Number(_norigmny) * (1 + Number(_ntaxrate) / 100);
            var _data = {};
            _data[norigbilltaxmny] = _cTaxMny2 ? _.round(_cTaxMny2, money) : _.round(0, money);
            editFormApi.setFieldsValue(_data);
        }
    },

    /**
     * 计算金额 = 单价 * 数量
     * @param editFormApi
     * @param fileds
     */
    calcHeadTaxMny: function calcHeadTaxMny(editFormApi, fileds) {
        var pk_taxrate = fileds.pk_taxrate,
            ntaxrate = fileds.ntaxrate,
            nnum = fileds.nnum,
            nprice = fileds.nprice,
            ntaxprice = fileds.ntaxprice,
            norigmny = fileds.norigmny,
            norigbilltaxmny = fileds.norigbilltaxmny,
            norigtaxmny = fileds.norigtaxmny;

        var _ntaxprice = ntaxprice && editFormApi.getFieldValue(ntaxprice);
        var _nnum = nnum && editFormApi.getFieldValue(nnum);
        var _cTaxMny = void 0;
        _ntaxprice = !_ntaxprice ? 0 : _ntaxprice;
        _nnum = !_nnum ? 0 : _nnum;
        if (norigbilltaxmny) {
            _cTaxMny = Number(_ntaxprice) * Number(_nnum);
            var _data = {};
            _data[norigbilltaxmny] = _cTaxMny ? _.round(_cTaxMny, money) : _.round(0, money);
            editFormApi.setFieldsValue(_data);
        }
    },

    /**
     * 计算单价 = 单价(无税) * (1 + 税率(%) / 100)
     * @param editFormApi
     * @param fileds
     */
    calcHeadTaxPrice: function calcHeadTaxPrice(editFormApi, fileds) {
        var pk_taxrate = fileds.pk_taxrate,
            ntaxrate = fileds.ntaxrate,
            nnum = fileds.nnum,
            nprice = fileds.nprice,
            ntaxprice = fileds.ntaxprice,
            norigmny = fileds.norigmny,
            norigbilltaxmny = fileds.norigbilltaxmny,
            norigtaxmny = fileds.norigtaxmny;

        var _nprice = nprice && editFormApi.getFieldValue(nprice);
        var _ntaxrate = ntaxrate && editFormApi.getFieldValue(ntaxrate);
        var _cTaxPrice = void 0;
        if (!_ntaxrate) {
            _ntaxrate = 0;
        }
        _nprice = !_nprice ? 0 : _nprice;
        if (ntaxprice) {
            _cTaxPrice = Number(_nprice) * (1 + Number(_ntaxrate) / 100);
            var _data = {};
            _data[ntaxprice] = _cTaxPrice ? _.round(_cTaxPrice, price) : _.round(0, price);
            editFormApi.setFieldsValue(_data);
        }
    },

    /**
     * 计算单价无税 = 单价 / (1 + 税率(%) / 100)
     * @param editFormApi
     * @param fileds
     */
    calcHeadPrice: function calcHeadPrice(editFormApi, fileds) {
        var pk_taxrate = fileds.pk_taxrate,
            ntaxrate = fileds.ntaxrate,
            nnum = fileds.nnum,
            nprice = fileds.nprice,
            ntaxprice = fileds.ntaxprice,
            norigmny = fileds.norigmny,
            norigbilltaxmny = fileds.norigbilltaxmny,
            norigtaxmny = fileds.norigtaxmny;

        var _ntaxprice = ntaxprice && editFormApi.getFieldValue(ntaxprice);
        var _ntaxrate = ntaxrate && editFormApi.getFieldValue(ntaxrate);
        var _cPrice = void 0;
        if (!_ntaxrate) {
            _ntaxrate = 0;
        }
        _ntaxprice = !_ntaxprice ? 0 : _ntaxprice;
        if (nprice) {
            _cPrice = Number(_ntaxprice) / (1 + Number(_ntaxrate) / 100);
            var _data = {};
            _data[nprice] = _cPrice ? _.round(_cPrice, price) : _.round(0, price);
            editFormApi.setFieldsValue(_data);
        }
    },

    /**
     * 计算金额(无税) = 单价(无税) * 数量
     * @param editFormApi
     * @param fileds
     */
    calcHeadMny: function calcHeadMny(editFormApi, fileds) {
        var pk_taxrate = fileds.pk_taxrate,
            ntaxrate = fileds.ntaxrate,
            nnum = fileds.nnum,
            nprice = fileds.nprice,
            ntaxprice = fileds.ntaxprice,
            norigmny = fileds.norigmny,
            norigbilltaxmny = fileds.norigbilltaxmny,
            norigtaxmny = fileds.norigtaxmny;

        var _nprice = nprice && editFormApi.getFieldValue(nprice);
        var _nnum = nnum && editFormApi.getFieldValue(nnum);
        var _cMny = void 0;
        _nprice = !_nprice ? 0 : _nprice;
        _nnum = !_nnum ? 0 : _nnum;
        if (norigmny) {
            _cMny = Number(_nprice) * Number(_nnum);
            var _data = {};
            _data[norigmny] = _cMny ? _.round(_cMny, money) : _.round(0, money);
            editFormApi.setFieldsValue(_data);
        }
    },

    /**
     * 计算金额(无税) = 金额 / (1 + 税率(%) / 100)
     * @param editFormApi
     * @param fileds
     */
    calcHeadMny2: function calcHeadMny2(editFormApi, fileds) {
        var pk_taxrate = fileds.pk_taxrate,
            ntaxrate = fileds.ntaxrate,
            nnum = fileds.nnum,
            nprice = fileds.nprice,
            ntaxprice = fileds.ntaxprice,
            norigmny = fileds.norigmny,
            norigbilltaxmny = fileds.norigbilltaxmny,
            norigtaxmny = fileds.norigtaxmny;

        var _norigbilltaxmny = norigbilltaxmny && editFormApi.getFieldValue(norigbilltaxmny);
        var _ntaxrate = ntaxrate && editFormApi.getFieldValue(ntaxrate);
        var _cMny2 = void 0;
        if (!_ntaxrate) {
            _ntaxrate = 0;
        }
        _norigbilltaxmny = !_norigbilltaxmny ? 0 : _norigbilltaxmny;
        if (norigmny) {
            _cMny2 = Number(_norigbilltaxmny) / (1 + Number(_ntaxrate) / 100);
            var _data = {};
            _data[norigmny] = _cMny2 ? _.round(_cMny2, money) : _.round(0, money);
            editFormApi.setFieldsValue(_data);
        }
    },

    /**
     * 计算税额 = 金额(无税) * 税率(%) / 100
     * @param editFormApi
     * @param fileds
     */
    calcHeadMnyTax: function calcHeadMnyTax(editFormApi, fileds) {
        var pk_taxrate = fileds.pk_taxrate,
            ntaxrate = fileds.ntaxrate,
            nnum = fileds.nnum,
            nprice = fileds.nprice,
            ntaxprice = fileds.ntaxprice,
            norigmny = fileds.norigmny,
            norigbilltaxmny = fileds.norigbilltaxmny,
            norigtaxmny = fileds.norigtaxmny;

        var _norigmny = norigmny && editFormApi.getFieldValue(norigmny);
        var _ntaxrate = ntaxrate && editFormApi.getFieldValue(ntaxrate);
        var _cMnyTax = void 0;
        if (!_ntaxrate) {
            _ntaxrate = 0;
        }
        _norigmny = !_norigmny ? 0 : _norigmny;
        if (norigtaxmny) {
            _cMnyTax = Number(_norigmny) * Number(_ntaxrate) / 100;
            var _data = {};
            _data[norigtaxmny] = _cMnyTax ? _.round(_cMnyTax, rate) : _.round(0, rate);
            editFormApi.setFieldsValue(_data);
        }
    },

    dealRelationFiled: function dealRelationFiled(fileds) {
        var _ref2 = fileds || {},
            pk_taxrate = _ref2.pk_taxrate,
            ntaxrate = _ref2.ntaxrate,
            nnum = _ref2.nnum,
            nprice = _ref2.nprice,
            ntaxprice = _ref2.ntaxprice,
            norigmny = _ref2.norigmny,
            norigbilltaxmny = _ref2.norigbilltaxmny,
            norigtaxmny = _ref2.norigtaxmny,
            nmny = _ref2.nmny,
            nbilltaxmny = _ref2.nbilltaxmny,
            ntaxmny = _ref2.ntaxmny;

        pk_taxrate && (fileds.pk_taxrate = pk_taxrate); // 税率参照
        ntaxrate && (fileds.ntaxrate = ntaxrate); // 税率
        nnum && (fileds.nnum = nnum); // 数量
        nprice && (fileds.nprice = nprice); // 单价(无税)
        ntaxprice && (fileds.ntaxprice = ntaxprice); // 单价(含税)
        norigmny && (fileds.norigmny = norigmny); // 金额(无税)(原币)
        nmny && (fileds.nmny = nmny); // 金额(无税)(组织本币)

        norigbilltaxmny && (fileds.norigbilltaxmny = norigbilltaxmny); // 金额(含税)(原币)
        nbilltaxmny && (fileds.nbilltaxmny = nbilltaxmny); // 金额(含税)(组织本币)

        norigtaxmny && (fileds.norigtaxmny = norigtaxmny); // 税额(原币)
        ntaxmny && (fileds.ntaxmny = ntaxmny); // 税额(组织本币)

        return fileds;
    }

};
module.exports = NccOtherCalculate;